@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2020-2021 BigML
+# Copyright 2020-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -45,6 +45,12 @@ anomaly.anomaly_score({"src_bytes": 350})
 
 import math
 
+try:
+    from pandas import DataFrame
+    PANDAS_READY = True
+except ImportError:
+    PANDAS_READY = False
+
 from bigml.predicate_utils.utils import OPERATOR_CODE, PREDICATE_INFO_LENGTH
 from bigml.predicate_utils.utils import apply_predicates
 from bigml.api import FINISHED
@@ -52,12 +58,15 @@ from bigml.api import get_status, get_api_connection, get_anomaly_id
 from bigml.basemodel import get_resource_dict
 from bigml.modelfields import ModelFields, NUMERIC
 from bigml.util import cast, use_cache, load
+from bigml.constants import OUT_NEW_HEADERS
 
 
 DEPTH_FACTOR = 0.5772156649
 PREDICATES_OFFSET = 3
 
+DFT_OUTPUTS = ["score"]
 
+#pylint: disable=locally-disabled,invalid-name
 def get_repeat_depth(population):
     """Computes the correction to depth used to normalize repeats
 
@@ -80,7 +89,7 @@ def build_tree(node, add_population=False):
     When the normalize_repeats flag is set to True, we need to add the
     population of the node: [weight, population, len(predicates), ...]
     """
-    outer = list()
+    outer = []
     outer.append(node.get('weight', 1))
     if add_population:
         outer.append(get_repeat_depth(node.get("population", 0)))
@@ -98,6 +107,7 @@ def build_tree(node, add_population=False):
 
 
 def build_predicates(node, encoded_node):
+    """Build the minified version of the predicate in a node"""
     predicates = node.get('predicates')
     if predicates and not (predicates is True or predicates == [True]):
         predicates = [x for x in predicates if x is not True]
@@ -192,6 +202,7 @@ class Anomaly(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.sample_size = None
         self.input_fields = None
         self.default_numeric_value = None
@@ -206,6 +217,7 @@ class Anomaly(ModelFields):
 
         if 'object' in anomaly and isinstance(anomaly['object'], dict):
             anomaly = anomaly['object']
+            self.dataset_id = anomaly.get('dataset')
             self.sample_size = anomaly.get('sample_size')
             self.input_fields = anomaly.get('input_fields')
             self.default_numeric_value = anomaly.get('default_numeric_value')
@@ -325,3 +337,43 @@ class Anomaly(ModelFields):
                     else field['summary'].get(self.default_numeric_value)
                 input_data[field_id] = default_value
         return input_data
+
+    def batch_predict(self, input_data_list, outputs=None, **kwargs):
+        """Creates a batch anomaly score for a list of inputs using the local
+        anomaly detector. Allows to define some output settings to decide the
+        name of the header used for the score in the result. To homogeneize
+        the behaviour of supervised batch_predict method, the outputs argument
+        accepts a dictionary with keys: "output_fields" and "output_headers".
+        In this case, output_fields is ignored, as only the score can be
+        obtained from the anomaly_score method, and only "output_headers" is
+        considered to allow changing the header associated to that new field.
+
+        :param input_data_list: List of input data to be predicted
+        :type input_data_list: list or Panda's dataframe
+        :param dict outputs: properties that define the headers and fields to
+                             be added to the input data
+        :return: the list of input data plus the predicted values
+        :rtype: list or Panda's dataframe depending on the input type in
+                input_data_list
+
+        """
+        if outputs is None:
+            outputs = {}
+        if outputs.get(OUT_NEW_HEADERS):
+            new_headers = outputs.get(OUT_NEW_HEADERS, DFT_OUTPUTS)
+        else:
+            new_headers = DFT_OUTPUTS
+
+        dataframe = False
+        if PANDAS_READY and isinstance(input_data_list, DataFrame):
+            dataframe = True
+            inner_data_list = input_data_list.to_dict('records')
+        else:
+            inner_data_list = input_data_list[:]
+        for input_data in inner_data_list:
+            prediction = {"score": self.anomaly_score(input_data, **kwargs)}
+            for index, key in enumerate(DFT_OUTPUTS):
+                input_data[new_headers[index]] = prediction[key]
+        if PANDAS_READY and dataframe:
+            return DataFrame.from_dict(inner_data_list)
+        return inner_data_list
