@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+#pylint: disable=too-many-ancestors,non-parent-init-called, unused-import, no-member
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -39,8 +40,8 @@ import json
 
 from bigml.bigmlconnection import BigMLConnection
 from bigml.domain import BIGML_PROTOCOL
-from bigml.constants import STORAGE, ALL_FIELDS
-from bigml.util import is_in_progress
+from bigml.constants import STORAGE, ALL_FIELDS, TINY_RESOURCE
+from bigml.util import is_in_progress, is_image
 from bigml.api_handlers.resourcehandler import ResourceHandlerMixin
 from bigml.api_handlers.sourcehandler import SourceHandlerMixin
 from bigml.api_handlers.datasethandler import DatasetHandlerMixin
@@ -132,7 +133,7 @@ from bigml.api_handlers.resourcehandler import (
     get_topic_distribution_id, get_batch_topic_distribution_id,
     get_time_series_id, get_forecast_id, get_deepnet_id, get_optiml_id,
     get_fusion_id, get_pca_id, get_projection_id, get_batch_projection_id,
-    get_configuration_id, get_linear_regression_id,
+    get_configuration_id, get_linear_regression_id, get_fields,
     get_script_id, get_execution_id, get_library_id, get_external_connector_id)
 
 
@@ -191,6 +192,55 @@ ID_GETTERS = {
 }
 
 
+PREDICTIONS = [PREDICTION_RE , PROJECTION_RE, ANOMALY_SCORE_RE,
+               CENTROID_RE, TOPIC_DISTRIBUTION_RE, ASSOCIATION_SET_RE]
+
+PREDICTION_LABELS = {
+    "anomalyscore": "score",
+    "topicdistribution": "topic distribution",
+    "associationset": "association set"}
+
+
+def get_resources_re(exceptions=None):
+    """Returning the patterns that correspond to a filtered subset of
+    resources.
+    """
+    if exceptions is None:
+        exceptions = {}
+    resources_re = list(RESOURCE_RE.values())
+    for res_re in exceptions:
+        resources_re.remove(res_re)
+    return resources_re
+
+
+NON_PREDICTIONS = get_resources_re(PREDICTIONS)
+
+
+def get_prediction_label(resource_id):
+    """Gets the label to be prepended to predictions according to their type"""
+    resource_type = get_resource_type(resource_id)
+    return PREDICTION_LABELS.get(resource_type, resource_type)
+
+
+#pylint: disable=locally-disabled,too-many-return-statements
+def get_prediction_attr(resource):
+    """Getting the attribute that contains the prediction, score, etc. """
+    if PREDICTION_RE.match(resource["resource"]):
+        return resource['object']['prediction'][
+            resource['object']['objective_fields'][0]]
+    if PROJECTION_RE.match(resource["resource"]):
+        return resource["object"]["projection"]["result"]
+    if ANOMALY_SCORE_RE.match(resource["resource"]):
+        return resource["object"]["score"]
+    if CENTROID_RE.match(resource["resource"]):
+        return resource["object"]["centroid_name"]
+    if TOPIC_DISTRIBUTION_RE.match(resource["resource"]):
+        return resource["object"]["topic_distribution"]["result"]
+    if ASSOCIATION_SET_RE.match(resource["resource"]):
+        return resource["object"]["association_set"]["result"]
+    return ""
+
+
 def count(listing):
     """Count of existing resources
 
@@ -213,30 +263,8 @@ def filter_kwargs(kwargs, list_of_keys, out=False):
             new_kwargs[key] = kwargs[key]
     return new_kwargs
 
-def get_fields(resource):
-    """Returns the field information in a resource dictionary structure
 
-    """
-    try:
-        resource_type = get_resource_type(resource)
-    except ValueError:
-        raise ValueError("Unknown resource structure. Failed to find"
-                         " a valid resource dictionary as argument.")
-
-    if resource_type in RESOURCES_WITH_FIELDS:
-        resource = resource.get('object', resource)
-        # fields structure
-        if resource_type in list(FIELDS_PARENT.keys()):
-            fields = resource[FIELDS_PARENT[resource_type]].get('fields', {})
-        else:
-            fields = resource.get('fields', {})
-
-        if resource_type == SAMPLE_PATH:
-            fields = {field['id']: field for field in fields}
-    return fields
-
-
-class BigML(ExternalConnectorHandlerMixin,
+class BigML(BigMLConnection,ExternalConnectorHandlerMixin,
             LinearRegressionHandlerMixin, BatchProjectionHandlerMixin,
             ProjectionHandlerMixin, PCAHandlerMixin,
             ConfigurationHandlerMixin, FusionHandlerMixin,
@@ -253,7 +281,7 @@ class BigML(ExternalConnectorHandlerMixin,
             AnomalyScoreHandlerMixin, AnomalyHandlerMixin,
             CentroidHandlerMixin, ClusterHandlerMixin, PredictionHandlerMixin,
             EnsembleHandlerMixin, ModelHandlerMixin, DatasetHandlerMixin,
-            SourceHandlerMixin, ResourceHandlerMixin, BigMLConnection):
+            SourceHandlerMixin, ResourceHandlerMixin):
     """Entry point to create, retrieve, list, update, and delete
     BigML resources.
 
@@ -301,12 +329,14 @@ class BigML(ExternalConnectorHandlerMixin,
         given by the organization administrator.
 
         """
-        BigMLConnection.__init__(self, username=username, api_key=api_key,
-                                 debug=debug,
-                                 set_locale=set_locale, storage=storage,
-                                 domain=domain, project=project,
-                                 organization=organization,
-                                 short_debug=short_debug)
+        # first BigMLConnection needs to exist
+        super().__init__(username=username, api_key=api_key,
+                         debug=debug,
+                         set_locale=set_locale, storage=storage,
+                         domain=domain, project=project,
+                         organization=organization,
+                         short_debug=short_debug)
+        # adding mixins properties
         ResourceHandlerMixin.__init__(self)
         SourceHandlerMixin.__init__(self)
         DatasetHandlerMixin.__init__(self)
@@ -376,6 +406,43 @@ class BigML(ExternalConnectorHandlerMixin,
             self.listers[resource_type] = getattr(self,
                                                   "list_%s" % method_name)
 
+    def prepare_image_fields(self, model_info, input_data):
+        """Creating a source for each image field used by the model
+        that is found in input_data
+
+        """
+        new_input_data = {}
+        new_input_data.update(input_data)
+        #pylint: disable=locally-disabled,broad-except
+        try:
+            fields = self.get_fields(model_info)
+            image_fields = [field_pair for field_pair in fields.items()
+                if field_pair[1]["optype"] == "image"]
+            for image_field, value in image_fields:
+                if image_field in input_data:
+                    key = image_field
+                    filename = input_data[key]
+                elif value["name"] in input_data:
+                    key = value["name"]
+                    filename = input_data[key]
+                source = self.create_source(filename)
+                source = self.check_resource(source,
+                                             query_string=TINY_RESOURCE,
+                                             raise_on_error=True)
+                new_input_data[key] = source["resource"]
+        except Exception:
+            # Predict Server does not return the fields info, so we infer
+            for field, value in input_data.items():
+                if isinstance(value, str) and os.path.isfile(value) and \
+                        is_image(value):
+                    source = self.create_source(value)
+                    source = self.check_resource(source,
+                                                 query_string=TINY_RESOURCE,
+                                                 raise_on_error=True)
+                    new_input_data[field] = source["resource"]
+
+        return new_input_data
+
     def create(self, resource_type, *args, **kwargs):
         """Create resources
 
@@ -451,20 +518,23 @@ class BigML(ExternalConnectorHandlerMixin,
 
         """
         info = "Connecting to:\n"
-        info += "    %s\n" % self.general_domain
-        if self.general_protocol != BIGML_PROTOCOL:
-            info += "    using %s protocol\n" % self.general_protocol
+        info += "    %s (%s)\n" % (self.domain.general_domain,
+                                   self.domain.api_version)
+        if self.domain.general_protocol != BIGML_PROTOCOL:
+            info += "    using %s protocol\n" % self.domain.general_protocol
         info += "    SSL verification %s\n" % (
-            "on" if self.verify else "off")
+            "on" if self.domain.verify else "off")
         short = "(shortened)" if self.short_debug else ""
         if self.debug:
             info += "    Debug on %s\n" % short
-        if self.general_domain != self.prediction_domain:
-            info += "    %s (predictions only)\n" % self.prediction_domain
-            if self.prediction_protocol != BIGML_PROTOCOL:
-                info += "    using %s protocol\n" % self.prediction_protocol
+        if self.domain.general_domain != self.domain.prediction_domain:
+            info += "    %s (predictions only)\n" % \
+                self.domain.prediction_domain
+            if self.domain.prediction_protocol != BIGML_PROTOCOL:
+                info += "    using %s protocol\n" % \
+                    self.domain.prediction_protocol
             info += "    SSL verification %s\n" % (
-                "on" if self.verify_prediction else "off")
+                "on" if self.domain.verify_prediction else "off")
 
         if self.project or self.organization:
             info += "    Scope info: %s\n" % \
@@ -533,7 +603,7 @@ class BigML(ExternalConnectorHandlerMixin,
                                               query_string=ALL_FIELDS)
         else:
             LOGGER.error("Wrong resource id")
-            return
+            return None
         # Tries to extract fields information from resource dict. If it fails,
         # a get remote call is used to retrieve the resource by id.
         fields = None
@@ -544,6 +614,7 @@ class BigML(ExternalConnectorHandlerMixin,
             fields = get_fields(resource)
         return fields
 
+    #pylint: disable=locally-disabled,no-self-use
     def pprint(self, resource, out=sys.stdout):
         """Pretty prints a resource or part of it.
 
@@ -554,26 +625,11 @@ class BigML(ExternalConnectorHandlerMixin,
                 and 'resource' in resource):
 
             resource_id = resource['resource']
-            if (SOURCE_RE.match(resource_id) or DATASET_RE.match(resource_id)
-                    or MODEL_RE.match(resource_id)
-                    or EVALUATION_RE.match(resource_id)
-                    or ENSEMBLE_RE.match(resource_id)
-                    or CLUSTER_RE.match(resource_id)
-                    or ANOMALY_RE.match(resource_id)
-                    or TOPIC_MODEL_RE.match(resource_id)
-                    or LOGISTIC_REGRESSION_RE.match(resource_id)
-                    or TIME_SERIES_RE.match(resource_id)
-                    or DEEPNET_RE.match(resource_id)
-                    or FUSION_RE.match(resource_id)
-                    or PCA_RE.match(resource_id)
-                    or LINEAR_REGRESSION_RE.match(resource_id)
-                    or OPTIML_RE.match(resource_id)):
+            if (any(getattr(res_re, "match")(resource_id) for res_re
+                    in NON_PREDICTIONS)):
                 out.write("%s (%s bytes)\n" % (resource['object']['name'],
                                                resource['object']['size']))
             elif PREDICTION_RE.match(resource['resource']):
-                objective_field_name = (
-                    resource['object']['fields'][
-                        resource['object']['objective_fields'][0]]['name'])
                 input_data = {}
                 for key, value in list(resource['object']['input_data'].items()):
                     try:
@@ -581,11 +637,15 @@ class BigML(ExternalConnectorHandlerMixin,
                     except KeyError:
                         name = key
                     input_data[name] = value
-
-                prediction = (
-                    resource['object']['prediction'][
-                        resource['object']['objective_fields'][0]])
-                out.write("%s for %s is %s\n" % (objective_field_name,
+                try:
+                    prediction_label = (
+                        resource['object']['fields'][
+                            resource['object']['objective_fields'][0]]['name'])
+                except IndexError:
+                    prediction_label = get_prediction_label(
+                        resource["resource"])
+                prediction = get_prediction_attr(resource)
+                out.write("%s for %s is %s\n" % (prediction_label,
                                                  input_data,
                                                  prediction))
             out.flush()
@@ -606,16 +666,18 @@ class BigML(ExternalConnectorHandlerMixin,
         status = get_status(resource)
         if status['code'] != UPLOADING:
             LOGGER.error("Wrong resource id")
-            return
+            return None
         return STATUSES[UPLOADING]
 
     def check_resource(self, resource,
-                       query_string='', wait_time=1):
+                       query_string='', wait_time=1, retries=None,
+                       raise_on_error=False):
         """Check resource method.
 
         """
         return check_resource(resource,
                               query_string=query_string, wait_time=wait_time,
+                              retries=retries, raise_on_error=raise_on_error,
                               api=self)
 
     def source_from_batch_prediction(self, batch_prediction, args=None):
@@ -629,6 +691,7 @@ class BigML(ExternalConnectorHandlerMixin,
             download_url = "%s%s%s%s" % (self.url, batch_prediction_id,
                                          DOWNLOAD_DIR, self.auth)
             return self._create_remote_source(download_url, args=args)
+        return None
 
     def retrieve_resource(self, resource_id, query_string=None,
                           check_local_fn=None, retries=None):
