@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+#pylint: disable=abstract-method,unused-import
 #
-# Copyright 2014-2021 BigML
+# Copyright 2014-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,6 +23,8 @@ import time
 import os
 import datetime
 import json
+import re
+import abc
 
 from xml.dom import minidom
 
@@ -31,9 +34,10 @@ from bigml.util import get_exponential_wait, get_status, is_status_final, \
     save, save_json
 from bigml.util import DFT_STORAGE
 from bigml.bigmlconnection import HTTP_OK, HTTP_ACCEPTED, HTTP_CREATED, \
-    LOGGER, DOWNLOAD_DIR
+    LOGGER, DOWNLOAD_DIR, HTTP_INTERNAL_SERVER_ERROR
 from bigml.constants import WAITING, QUEUED, STARTED, IN_PROGRESS, \
     SUMMARIZED, FINISHED, UPLOADING, FAULTY, UNKNOWN, RUNNABLE
+from bigml.exceptions import FaultyResourceError
 
 # Minimum query string to get model fields
 TINY_RESOURCE = "full=false"
@@ -79,6 +83,41 @@ def get_resource(resource_type, resource):
     raise ValueError("%s is not a valid resource ID." % resource)
 
 
+def get_id(pure_id):
+    """Returns last part or a resource ID.
+
+    """
+    if isinstance(pure_id, str):
+        pure_id = re.sub(r'^[^/]*/(%s)' % c.ID_PATTERN, r'\1', pure_id)
+        if c.ID_RE.match(pure_id):
+            return pure_id
+    raise ValueError("%s is not a valid ID." % pure_id)
+
+
+def get_fields(resource):
+    """Returns the field information in a resource dictionary structure
+
+    """
+    try:
+        resource_type = get_resource_type(resource)
+    except ValueError:
+        raise ValueError("Unknown resource structure. Failed to find"
+                         " a valid resource dictionary as argument.")
+
+    if resource_type in c.RESOURCES_WITH_FIELDS:
+        resource = resource.get('object', resource)
+        # fields structure
+        if resource_type in list(c.FIELDS_PARENT.keys()) and \
+                c.FIELDS_PARENT[resource_type] is not None:
+            fields = resource[c.FIELDS_PARENT[resource_type]].get('fields', {})
+        else:
+            fields = resource.get('fields', {})
+
+        if resource_type == c.SAMPLE_PATH:
+            fields = {field['id']: field for field in fields}
+    return fields
+
+
 def resource_is_ready(resource):
     """Checks a fully fledged resource structure and returns True if finished.
 
@@ -382,21 +421,32 @@ def get_resource_id(resource):
             resource_re.match(resource) for _, resource_re
             in list(c.RESOURCE_RE.items())):
         return resource
-    return
+    return None
 
 
-def exception_on_error(resource):
-    """Raises exception if resource has error
+def exception_on_error(resource, logger=None):
+    """Raises exception if the resource has an error. The error can be
+    due to a problem in the API call to retrieve it or because the
+    resource is FAULTY.
 
     """
     if resource.get('error') is not None:
-        raise Exception(resource.get('error', \
-            {}).get('status', {}).get('message'))
+        # http error calling the API
+        message = "API connection problem - %s" % resource.get('error', \
+            {}).get('status', {}).get('message')
+        if logger is not None:
+            logger.error(message)
+        raise Exception(message)
     if resource.get('object', resource).get('status', {}).get('error') \
             is not None:
+        # Faulty resource problem
         status = resource.get('object', resource).get( \
             'status', {})
-        raise Exception(status.get('cause', status).get('message'))
+        message = "Faulty resource %s - %s" % (resource["resource"],
+            status.get('cause', status).get('message'))
+        if logger is not None:
+            logger.error(message)
+        raise FaultyResourceError(message)
 
 
 def check_resource(resource, get_method=None, query_string='', wait_time=1,
@@ -425,16 +475,25 @@ def check_resource(resource, get_method=None, query_string='', wait_time=1,
     if debug:
         print("Checking resource: %s" % resource_id)
     kwargs = {'query_string': query_string}
-
-    if get_method is None and hasattr(api, 'get_resource'):
+    if hasattr(api, 'shared_ref') or (get_method is None and
+            hasattr(api, 'get_resource')):
         get_method = api.get_resource
     elif get_method is None:
         raise ValueError("You must supply either the get_method or the api"
                          " connection info to retrieve the resource")
+    if not isinstance(resource, dict) and not http_ok(resource):
+        resource = resource_id
+
     if isinstance(resource, str):
         if debug:
             print("Getting resource %s" % resource_id)
         resource = get_method(resource_id, **kwargs)
+        if not http_ok(resource):
+            if raise_on_error:
+                raise Exception("API connection problem: %s" %
+                    json.dumps(resource))
+            return resource
+
     counter = 0
     elapsed = 0
     while retries is None or counter < retries:
@@ -480,6 +539,13 @@ def check_resource(resource, get_method=None, query_string='', wait_time=1,
         if debug:
             print("Getting only status for resource %s" % resource_id)
         resource = get_method(resource, **tiny_kwargs)
+        if not http_ok(resource):
+            resource["resource"] = resource_id
+            if raise_on_error:
+                raise Exception("API connection problem: %s" %
+                    json.dumps(resource))
+            return resource
+
     if raise_on_error:
         exception_on_error(resource)
     return resource
@@ -491,16 +557,22 @@ def http_ok(resource):
     """
     if 'code' in resource:
         return resource['code'] in [HTTP_OK, HTTP_CREATED, HTTP_ACCEPTED]
+    return False
 
 
-
-class ResourceHandlerMixin():
+class ResourceHandlerMixin(metaclass=abc.ABCMeta):
     """This class is used by the BigML class as
        a mixin that provides the get method for all kind of
        resources and auxiliar utilities to check their status. It should not
        be instantiated independently.
 
     """
+    @abc.abstractmethod
+    def prepare_image_fields(self, model_info, input_data):
+        """This is an abstract method that should be implemented in the API
+        final class to create sources for the image fields used in the model
+
+        """
 
     def get_resource(self, resource, **kwargs):
         """Retrieves a remote resource.
@@ -518,8 +590,14 @@ class ResourceHandlerMixin():
             raise ValueError("A resource id or structure is needed.")
         resource_id = get_resource_id(resource)
 
+        # adding the shared_ref if the API connection object has one
+        if hasattr(self, "shared_ref"):
+            kwargs.update({"shared_ref": self.shared_ref})
+
         if resource_id:
+            kwargs.update({"resource_id": resource_id})
             return self._get("%s%s" % (self.url, resource_id), **kwargs)
+        return None
 
     def update_resource(self, resource, changes, **kwargs):
         """Updates a remote resource.
@@ -533,6 +611,7 @@ class ResourceHandlerMixin():
             raise Exception("Failed to update %s. Only correctly finished "
                             "resources can be updated. Please, check "
                             "the resource status." % resource_id)
+        kwargs.update({"resource_id": resource_id})
         body = json.dumps(changes)
         return self._update("%s%s" % (self.url, resource_id), body, **kwargs)
 
@@ -543,6 +622,7 @@ class ResourceHandlerMixin():
         resource_id = get_resource_id(resource)
         if resource_id:
             return self._delete("%s%s" % (self.url, resource_id), **kwargs)
+        return None
 
     def _download_resource(self, resource, filename, retries=10):
         """Download CSV information from downloadable resources
@@ -558,11 +638,13 @@ class ResourceHandlerMixin():
                               filename=filename,
                               retries=retries)
 
+    #pylint: disable=locally-disabled,invalid-name
     def ok(self, resource, query_string='', wait_time=1,
            max_requests=None, raise_on_error=False, retries=None,
            error_retries=None, max_elapsed_estimate=float('inf'), debug=False):
         """Waits until the resource is finished or faulty, updates it and
-           returns True on success
+           returns True when a finished resource is correctly retrieved
+           and False if the retrieval fails or the resource is faulty.
 
              resource: (map) Resource structure
              query_string: (string) Filters used on the resource attributes
@@ -578,42 +660,53 @@ class ResourceHandlerMixin():
              debug: (boolean) Whether to print traces for every get call
 
         """
-        if http_ok(resource):
-            try:
-                resource.update(check_resource( \
-                    resource,
-                    query_string=query_string,
-                    wait_time=wait_time,
-                    retries=max_requests,
-                    max_elapsed_estimate=max_elapsed_estimate,
-                    raise_on_error=raise_on_error,
-                    api=self,
-                    debug=debug))
-                if resource['error'] and resource['error'].get( \
-                        'status', {}).get('type') == c.TRANSIENT and \
-                        error_retries is not None and error_retries > 0:
-                    return self.ok(resource, query_string, wait_time,
-                                   max_requests, raise_on_error, retries,
-                                   error_retries - 1, max_elapsed_estimate,
-                                   debug)
-                return True
-            except Exception as err:
-                if error_retries is not None and error_retries > 0:
-                    return self.ok(resource, query_string, wait_time,
-                                   max_requests, raise_on_error, retries,
-                                   error_retries - 1,
-                                   max_elapsed_estimate,
-                                   debug)
-                LOGGER.error("The resource info for %s couldn't"
-                             " be retrieved", resource["resource"])
-                if raise_on_error:
-                    exception_on_error({"resource": resource["resource"],
-                                        "error": err})
-        else:
-            LOGGER.error("The resource %s couldn't be retrieved: %s",
-                         resource["location"], resource['error'])
+        def maybe_retrying(resource, error_retries, new_resource=None):
+            """Retrying retrieval if it's due to a transient error """
+            if new_resource is None:
+                new_resource = resource
+            else:
+                new_resource.update({"object": resource["object"]})
+            if new_resource.get('error', {}).get(
+                    'status', {}).get('type') == c.TRANSIENT \
+                    and error_retries is not None and error_retries > 0:
+                time.sleep(wait_time)
+                return self.ok(resource, query_string, wait_time,
+                               max_requests, raise_on_error, retries,
+                               error_retries - 1, max_elapsed_estimate,
+                               debug)
+            resource.update(new_resource)
             if raise_on_error:
-                exception_on_error(resource)
+                exception_on_error(resource, logger=LOGGER)
+            return False
+
+        new_resource = check_resource( \
+            resource,
+            query_string=query_string,
+            wait_time=wait_time,
+            retries=max_requests,
+            max_elapsed_estimate=max_elapsed_estimate,
+            raise_on_error=False, # we don't raise on error to update always
+            api=self,
+            debug=debug)
+
+
+        if http_ok(new_resource):
+            resource.update(new_resource)
+            # try to recover from transient errors
+            if resource["error"] is not None:
+                return maybe_retrying(resource, error_retries)
+
+            #pylint: disable=locally-disabled,bare-except
+            if raise_on_error:
+                exception_on_error(resource, logger=LOGGER)
+            else:
+                try:
+                    exception_on_error(resource)
+                except:
+                    return False
+            return True
+        return maybe_retrying(resource, error_retries,
+                              new_resource=new_resource)
 
     def _set_create_from_datasets_args(self, datasets, args=None,
                                        wait_time=3, retries=10, key=None):
@@ -813,7 +906,6 @@ class ResourceHandlerMixin():
                                  " can be exported to PMML.")
 
         resource_id = get_resource_id(resource)
-
         if resource_id:
             if pmml:
                 # only models with no text fields can be exported
@@ -845,15 +937,24 @@ class ResourceHandlerMixin():
                 filename = os.path.join( \
                     file_dir, resource_id.replace("/", "_"))
             if resource_type in COMPOSED_RESOURCES:
+                # inner models in composed resources need the shared reference
+                # to be downloaded
+                if resource.startswith("shared"):
+                    kwargs.update(
+                        {"shared_ref": resource_id.replace("shared/", "")})
+                elif "shared_ref" in kwargs and not resource.startswith("shared"):
+                    kwargs["shared_ref"] = "%s,%s" % (kwargs["shared_ref"],
+                                                      resource_id)
                 for component_id in resource_info["object"]["models"]:
-                    # for weighted fusions we need to retrieve the component
-                    # ID
+                    # for weighted fusions we need to retrieve the component ID
                     if isinstance(component_id, dict):
                         component_id = component_id['id']
+                    component_filename = os.path.join(
+                        os.path.dirname(filename),
+                        component_id.replace("/", "_"))
                     self.export( \
                         component_id,
-                        filename=os.path.join(os.path.dirname(filename),
-                                              component_id.replace("/", "_")),
+                        filename=component_filename,
                         pmml=pmml,
                         **kwargs)
             if kwargs.get("query_string") and \
