@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -105,7 +105,7 @@ class Association(ModelFields):
     that can be used to extract associations information.
 
     """
-
+    #pylint: disable=locally-disabled,access-member-before-definition
     def __init__(self, association, api=None, cache_get=None):
 
 
@@ -119,6 +119,7 @@ class Association(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.complement = None
         self.discretization = {}
         self.default_numeric_value = None
@@ -140,6 +141,7 @@ class Association(ModelFields):
 
         if 'object' in association and isinstance(association['object'], dict):
             association = association['object']
+            self.dataset_id = association.get('dataset')
 
         if 'associations' in association and \
                 isinstance(association['associations'], dict):
@@ -445,6 +447,7 @@ class Association(ModelFields):
         for metric in ASSOCIATION_METRICS:
             out.write("\n\nTop %s by %s:\n\n" % (
                 limit, METRIC_LITERALS[metric]))
+            #pylint: disable=locally-disabled,cell-var-from-loop
             top_rules = sorted(rules, key=lambda x: getattr(x, metric),
                                reverse=True)[0: limit * 2]
             out_rules = []
