@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2013-2021 BigML
+# Copyright 2013-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -24,14 +24,15 @@ is used for local predictions.
 import logging
 import sys
 import json
+import os
 
 from bigml.api import FINISHED
 from bigml.api import get_status, get_model_id, ID_GETTERS, \
     get_api_connection
 from bigml.util import utf8
 from bigml.util import DEFAULT_LOCALE
-from bigml.modelfields import ModelFields, check_model_structure, \
-    check_model_fields
+from bigml.modelfields import ModelFields, check_resource_structure, \
+    check_resource_fields
 from bigml.api_handlers.resourcehandler import resource_is_ready
 
 LOGGER = logging.getLogger('BigML')
@@ -84,30 +85,30 @@ def print_importance(instance, out=sys.stdout):
             round(importance, 4) * 100)))
         count += 1
 
-
-def check_local_but_fields(model):
-    """Whether the information in `model` is enough to use it locally
+#pylint: disable=locally-disabled,broad-except
+def check_local_but_fields(resource):
+    """Whether the information in `resource` is enough to use it locally
        except for the fields section
 
     """
     try:
-
-        return resource_is_ready(model) and \
-            check_model_structure(model)
+        return resource_is_ready(resource) and \
+            check_resource_structure(resource)
     except Exception:
         return False
 
-
-def check_local_info(model):
+#pylint: disable=locally-disabled,broad-except
+def check_local_info(resource):
     """Whether the information in `model` is enough to use it locally
 
     """
     try:
-        return check_local_but_fields(model) and \
-            check_model_fields(model)
+        return check_local_but_fields(resource) and \
+            check_resource_fields(resource)
     except Exception:
         return False
 
+
 def get_resource_dict(resource, resource_type, api=None,
                       no_check_fields=False):
     """Extracting the resource JSON info as a dict from the first argument of
@@ -124,6 +125,7 @@ def get_resource_dict(resource, resource_type, api=None,
     # the string can be a path to a JSON file
     if isinstance(resource, str):
         try:
+            resource_path = resource
             with open(resource) as resource_file:
                 resource = json.load(resource_file)
                 resource_id = get_id(resource)
@@ -131,6 +133,10 @@ def get_resource_dict(resource, resource_type, api=None,
                     raise ValueError("The JSON file does not seem"
                                      " to contain a valid BigML %s"
                                      " representation." % resource_type)
+                # keeping the path to the main file as storage folder for
+                # related files
+                storage = os.path.dirname(resource_path)
+                api.storage = storage
         except IOError:
             # if it is not a path, it can be a model id
             resource_id = get_id(resource)
@@ -152,9 +158,9 @@ def get_resource_dict(resource, resource_type, api=None,
     check_fn = check_local_but_fields if no_check_fields else \
         check_local_info
 
-    if isinstance(resource, dict) and not check_fn( \
+    if isinstance(resource, dict) and not check_fn(
             resource):
-        # if the fields used by the model are not
+        # if the fields used by the resource are not
         # available, use only ID to retrieve it again
         resource = get_id(resource)
         resource_id = resource
@@ -191,10 +197,12 @@ class BaseModel(ModelFields):
         fields: fields dict (used in ensembles where fields info can be shared)
         checked: boolean that avoids rechecking the model structure when it
                  has already been checked previously in a derived class
+        operation_settings: operation thresholds for the classification model
 
     """
 
-    def __init__(self, model, api=None, fields=None, checked=True):
+    def __init__(self, model, api=None, fields=None, checked=True,
+                 operation_settings=None):
 
         check_fn = check_local_but_fields if fields is not None else \
             check_local_info
@@ -223,33 +231,37 @@ class BaseModel(ModelFields):
         if 'model' in model and isinstance(model['model'], dict):
             status = get_status(model)
             if 'code' in status and status['code'] == FINISHED:
+                model_fields = None
                 if (fields is None and ('model_fields' in model['model'] or
                                         'fields' in model['model'])):
-                    fields = model['model'].get('model_fields',
-                                                model['model'].get('fields',
-                                                                   []))
-                    # model_fields doesn't contain the datetime fields
-                    fields.update(datetime_fields(model['model'].get('fields',
-                                                                     {})))
+                    # models might use less fields that provided
+                    model_fields = model['model'].get('model_fields')
+                    fields = model['model'].get('fields', {})
                     # pagination or exclusion might cause a field not to
                     # be in available fields dict
-                    if not all(key in model['model']['fields']
-                               for key in list(fields.keys())):
-                        raise Exception("Some fields are missing"
-                                        " to generate a local model."
-                                        " Please, provide a model with"
-                                        " the complete list of fields.")
-                    for field in fields:
-                        field_info = model['model']['fields'][field]
-                        if 'summary' in field_info:
-                            fields[field]['summary'] = field_info['summary']
-                        fields[field]['name'] = field_info['name']
+                    if model_fields:
+                        if not all(key in fields
+                                   for key in list(model_fields.keys())):
+                            raise Exception("Some fields are missing"
+                                            " to generate a local model."
+                                            " Please, provide a model with"
+                                            " the complete list of fields.")
+                        for field in model_fields:
+                            field_info = fields[field]
+                            if 'summary' in field_info:
+                                model_fields[field]['summary'] = field_info[
+                                    'summary']
+                                model_fields[field]['name'] = field_info[
+                                    'name']
                 objective_field = model['objective_fields']
                 missing_tokens = model['model'].get('missing_tokens')
 
                 ModelFields.__init__(
-                    self, fields, objective_id=extract_objective(objective_field),
-                    missing_tokens=missing_tokens)
+                    self, fields, objective_id=extract_objective(
+                        objective_field),
+                        missing_tokens=missing_tokens,
+                        operation_settings=operation_settings,
+                        model_fields=model_fields)
                 self.description = model['description']
                 self.field_importance = model['model'].get('importance',
                                                            None)
