@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2021 BigML
+# Copyright 2014-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -40,11 +40,10 @@ except ImportError:
 
 import bigml.constants as c
 
-from bigml.util import (check_dir,
-                        maybe_save, get_exponential_wait)
+from bigml.util import check_dir, maybe_save, get_exponential_wait
 from bigml.util import DEFAULT_LOCALE
 from bigml.domain import Domain
-from bigml.domain import DEFAULT_DOMAIN, BIGML_PROTOCOL
+from bigml.domain import DEFAULT_DOMAIN
 
 
 LOG_FORMAT = '%(asctime)-15s: %(message)s'
@@ -55,7 +54,7 @@ LOGGER.addHandler(CONSOLE)
 
 
 # Base URL
-BIGML_URL = '%s://%s/andromeda/'
+BIGML_URL = '%s://%s/%s'
 
 DOWNLOAD_DIR = '/download'
 
@@ -127,6 +126,7 @@ def json_load(content):
 # Patch for requests
 #
 ##############################################################################
+#pylint: disable=locally-disabled,broad-except,used-before-assignment
 def patch_requests(short_debug):
     """ Monkey patches requests to get debug output.
 
@@ -137,13 +137,17 @@ def patch_requests(short_debug):
         """
         response = original_request(method, url, **kwargs)
         logging.debug("Data: %s", response.request.body)
-        response_content = "Download status is %s" % response.status_code \
-            if "download" in url else \
-            response.content
+        try:
+            response_content = "Download status is %s" % response.status_code \
+                if "download" in url else \
+                json.dumps(json.loads(response.content), indent=4)
+        except Exception:
+            response_content = response.content
         response_content = response_content[0:256] if short_debug else \
         response_content
         logging.debug("Response: %s\n", response_content)
         return response
+
     original_request = requests.api.request
     requests.api.request = debug_request
 
@@ -215,16 +219,19 @@ class BigMLConnection():
             try:
                 username = os.environ['BIGML_USERNAME']
             except KeyError:
-                raise AttributeError("Cannot find BIGML_USERNAME in"
-                                     " your environment")
+                if storage is None:
+                    raise AttributeError("Cannot find BIGML_USERNAME in"
+                                         " your environment")
 
         if api_key is None:
             try:
                 api_key = os.environ['BIGML_API_KEY']
             except KeyError:
-                raise AttributeError("Cannot find BIGML_API_KEY in"
-                                     " your environment")
+                if storage is None:
+                    raise AttributeError("Cannot find BIGML_API_KEY in"
+                                         " your environment")
 
+        self.username = username
         self.auth = "?username=%s;api_key=%s;" % (username, api_key)
         self.project = None
         self.organization = None
@@ -234,12 +241,7 @@ class BigMLConnection():
             self.organization = organization
         self.debug = debug
         self.short_debug = short_debug
-        self.general_domain = None
-        self.general_protocol = None
-        self.prediction_domain = None
-        self.prediction_protocol = None
-        self.verify = None
-        self.verify_prediction = None
+        self.domain = None
         self.url = None
         self.prediction_base_url = None
 
@@ -247,7 +249,7 @@ class BigMLConnection():
 
         # if verify is not set, we capture warnings to avoid `requests` library
         # warnings: InsecurePlatformWarning
-        logging.captureWarnings(not self.verify)
+        logging.captureWarnings(not self.domain.verify)
         if set_locale:
             locale.setlocale(locale.LC_ALL, DEFAULT_LOCALE)
         self.storage = assign_dir(storage)
@@ -271,15 +273,14 @@ class BigMLConnection():
         elif not isinstance(domain, Domain):
             raise ValueError("The domain must be set using a Domain object.")
         # Setting the general and prediction domain options
-        self.general_domain = domain.general_domain
-        self.general_protocol = domain.general_protocol
-        self.prediction_domain = domain.prediction_domain
-        self.prediction_protocol = domain.prediction_protocol
-        self.verify = domain.verify
-        self.verify_prediction = domain.verify_prediction
-        self.url = BIGML_URL % (BIGML_PROTOCOL, self.general_domain)
+        self.domain = domain
+        api_version = "%s/" % self.domain.api_version if \
+            self.domain.api_version != "" else ""
+        self.url = BIGML_URL % (self.domain.general_protocol,
+                                self.domain.general_domain,
+                                api_version)
         self.prediction_base_url = BIGML_URL % (
-            self.prediction_protocol, self.prediction_domain)
+            self.domain.prediction_protocol, self.domain.prediction_domain, "")
 
 
     def _add_credentials(self, url, organization=False, shared_auth=None):
@@ -347,7 +348,7 @@ class BigMLConnection():
         # downloaded.
         code = HTTP_ACCEPTED
         if verify is None:
-            verify = self.verify
+            verify = self.domain.verify
 
         url = self._add_credentials(url, organization=organization)
         body = self._add_project(body, not organization)
@@ -408,7 +409,8 @@ class BigMLConnection():
                           location, resource, error)
 
     def _get(self, url, query_string='',
-             shared_username=None, shared_api_key=None, organization=None):
+             shared_username=None, shared_api_key=None, organization=None,
+             shared_ref=None, resource_id=None):
         """Retrieves a remote resource.
 
         Uses HTTP GET to retrieve a BigML `url`.
@@ -422,7 +424,6 @@ class BigMLConnection():
 
         """
         code = HTTP_INTERNAL_SERVER_ERROR
-        resource_id = None
         location = url
         resource = None
         error = {
@@ -438,13 +439,15 @@ class BigMLConnection():
             kwargs.update({"shared_auth": auth})
 
         url = self._add_credentials(url, **kwargs) + query_string
+        if shared_ref is not None:
+            url = "%sshared_ref=%s" % (url, shared_ref)
         if GAE_ENABLED:
             try:
                 req_options = {
                     'url': url,
                     'method': urlfetch.GET,
                     'headers': ACCEPT_JSON,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -456,7 +459,7 @@ class BigMLConnection():
         else:
             try:
                 response = requests.get(url, headers=ACCEPT_JSON,
-                                        verify=self.verify)
+                                        verify=self.domain.verify)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -528,7 +531,7 @@ class BigMLConnection():
                     'url': url,
                     'method': urlfetch.GET,
                     'headers': ACCEPT_JSON,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -543,7 +546,7 @@ class BigMLConnection():
         else:
             try:
                 response = requests.get(url, headers=ACCEPT_JSON,
-                                        verify=self.verify)
+                                        verify=self.domain.verify)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -579,7 +582,7 @@ class BigMLConnection():
             'objects': resources,
             'error': error}
 
-    def _update(self, url, body, organization=None):
+    def _update(self, url, body, organization=None, resource_id=None):
         """Updates a remote resource.
 
         Uses PUT to update a BigML resource. Only the new fields that
@@ -595,7 +598,6 @@ class BigMLConnection():
 
         """
         code = HTTP_INTERNAL_SERVER_ERROR
-        resource_id = None
         location = url
         resource = None
         error = {
@@ -612,7 +614,7 @@ class BigMLConnection():
                     'method': urlfetch.PUT,
                     'headers': SEND_JSON,
                     'payload': body,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -625,7 +627,7 @@ class BigMLConnection():
             try:
                 response = requests.put(url,
                                         headers=SEND_JSON,
-                                        data=body, verify=self.verify)
+                                        data=body, verify=self.domain.verify)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -655,7 +657,8 @@ class BigMLConnection():
         return maybe_save(resource_id, self.storage, code,
                           location, resource, error)
 
-    def _delete(self, url, query_string='', organization=None):
+    def _delete(self, url, query_string='', organization=None,
+                resource_id=None):
         """Permanently deletes a remote resource.
 
         If the request is successful the status `code` will be HTTP_NO_CONTENT
@@ -676,7 +679,7 @@ class BigMLConnection():
                 req_options = {
                     'url': url,
                     'method': urlfetch.DELETE,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -685,10 +688,11 @@ class BigMLConnection():
                 error["status"]["type"] = c.TRANSIENT
                 return {
                     'code': code,
+                    'resource': resource_id,
                     'error': error}
         else:
             try:
-                response = requests.delete(url, verify=self.verify)
+                response = requests.delete(url, verify=self.domain.verify)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -696,6 +700,7 @@ class BigMLConnection():
                 error["status"]["type"] = c.TRANSIENT
                 return {
                     'code': code,
+                    'resource': resource_id,
                     'error': error}
         try:
             code = response.status_code
@@ -717,6 +722,7 @@ class BigMLConnection():
 
         return {
             'code': code,
+            'resource': resource_id,
             'error': error}
 
     def _download(self, url, filename=None, wait_time=10, retries=10,
@@ -739,7 +745,7 @@ class BigMLConnection():
                 req_options = {
                     'url': self._add_credentials(url),
                     'method': urlfetch.GET,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -749,7 +755,8 @@ class BigMLConnection():
         else:
             try:
                 response = requests.get(self._add_credentials(url),
-                                        verify=self.verify, stream=True)
+                                        verify=self.domain.verify,
+                                        stream=True)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -857,7 +864,7 @@ class BigMLConnection():
                     'url': url,
                     'method': urlfetch.GET,
                     'headers': ACCEPT_JSON,
-                    'validate_certificate': self.verify
+                    'validate_certificate': self.domain.verify
                 }
                 response = urlfetch.fetch(**req_options)
             except urlfetch.Error as exception:
@@ -870,7 +877,7 @@ class BigMLConnection():
         else:
             try:
                 response = requests.get(url, headers=ACCEPT_JSON,
-                                        verify=self.verify)
+                                        verify=self.domain.verify)
             except (requests.ConnectionError,
                     requests.Timeout,
                     requests.RequestException) as exc:
@@ -915,6 +922,9 @@ class BigMLConnection():
             elif ('code' in resource
                   and 'status' in resource):
                 error_info = resource
+            resource_id = resource.get("resource")
+        else:
+            resource_id = resource
         if error_info is not None and 'code' in error_info:
             code = error_info['code']
             if ('status' in error_info and
@@ -925,26 +935,27 @@ class BigMLConnection():
                     error += ": %s" % extra
             if code == HTTP_NOT_FOUND and method == 'get':
                 alternate_message = ''
-                if self.general_domain != DEFAULT_DOMAIN:
+                if self.domain.general_domain != DEFAULT_DOMAIN:
                     alternate_message = (
                         '- The %s was not created in %s.\n' % (
-                            resource_type, self.general_domain))
+                            resource_type, self.domain.general_domain))
                 error += (
                     '\nCouldn\'t find a %s matching the given'
-                    ' id in %s. The most probable causes are:\n\n%s'
+                    ' id (%s) in %s. The most probable causes are:\n\n%s'
                     '- A typo in the %s\'s id.\n'
                     '- The %s id cannot be accessed with your credentials'
                     ' or was not created in %s.\n'
                     '\nDouble-check your %s and'
                     ' credentials info and retry.' % (
-                        resource_type, self.general_domain,
+                        resource_type, resource_id, self.domain.general_domain,
                         alternate_message, resource_type,
-                        resource_type, self.general_domain, resource_type))
+                        resource_type, self.domain.general_domain,
+                        resource_type))
                 return error
             if code == HTTP_UNAUTHORIZED:
                 error += ('\nDouble-check your credentials and the general'
                           ' domain your account is registered with (currently'
-                          ' using %s), please.' % self.general_domain)
+                          ' using %s), please.' % self.domain.general_domain)
                 return error
             if code == HTTP_BAD_REQUEST:
                 error += '\nDouble-check the arguments for the call, please.'
@@ -954,8 +965,9 @@ class BigMLConnection():
                           ' requests for a while before resuming.')
                 return error
             if code == HTTP_PAYMENT_REQUIRED:
-                error += ('\nYou\'ll need to buy some more credits to perform'
-                          ' the chosen action')
+                error += ('\nThis operation exceeds your subscription limits.'
+                          ' Please, upgrade your subscription, reduce the '
+                          'dataset size or wait for a running task to finish.')
                 return error
 
         return "Invalid %s structure:\n\n%s" % (resource_type, resource)
