@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2021 BigML
+# Copyright 2014-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a Cluster to make predictions (centroids) locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your clusters
 offline.
 
@@ -45,6 +45,11 @@ import re
 import csv
 import codecs
 
+try:
+    from pandas import DataFrame
+    PANDAS_READY = True
+except ImportError:
+    PANDAS_READY = False
 
 from bigml.api import FINISHED
 from bigml.api import get_status, get_api_connection, get_cluster_id
@@ -55,7 +60,7 @@ from bigml.generators.model import print_distribution
 from bigml.predicate import TM_TOKENS, TM_FULL_TERM
 from bigml.modelfields import ModelFields
 from bigml.io import UnicodeWriter
-
+from bigml.constants import OUT_NEW_FIELDS, OUT_NEW_HEADERS
 
 
 LOGGER = logging.getLogger('BigML')
@@ -68,6 +73,8 @@ INTERCENTROID_MEASURES = [('Minimum', min),
                           ('Maximum', max)]
 GLOBAL_CLUSTER_LABEL = 'Global'
 
+DFT_OUTPUTS = ["centroid_name", "distance"]
+
 
 def parse_terms(text, case_sensitive=True):
     """Returns the list of parsed terms
@@ -157,6 +164,7 @@ class Cluster(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.cluster_global = None
         self.total_ss = None
         self.within_ss = None
@@ -182,6 +190,7 @@ class Cluster(ModelFields):
 
         if 'object' in cluster and isinstance(cluster['object'], dict):
             cluster = cluster['object']
+            self.dataset_id = cluster.get('dataset')
 
         if 'clusters' in cluster and isinstance(cluster['clusters'], dict):
             status = get_status(cluster)
@@ -227,8 +236,8 @@ class Cluster(ModelFields):
                 missing_tokens = cluster['clusters'].get('missing_tokens')
                 ModelFields.__init__(self, fields,
                                      missing_tokens=missing_tokens)
-                if not all([field_id in self.fields for
-                            field_id in self.scales]):
+                if not all(field_id in self.fields for
+                            field_id in self.scales):
                     raise Exception("Some fields are missing"
                                     " to generate a local cluster."
                                     " Please, provide a cluster with"
@@ -315,6 +324,7 @@ class Cluster(ModelFields):
                     unique_terms[field_id] = input_data_field
                 del input_data[field_id]
         # the same for items fields
+        #pylint: disable=locally-disabled,consider-using-dict-items
         for field_id in self.item_analysis:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
@@ -574,6 +584,7 @@ class Cluster(ModelFields):
             return rows
         with UnicodeWriter(file_name) as writer:
             writer.writerows(rows)
+        return file_name
 
     def summarize(self, out=sys.stdout):
         """Prints a summary of the cluster info
@@ -631,6 +642,52 @@ class Cluster(ModelFields):
                     out.write("%s%s: %s\n" % (INDENT * 2, measure, result))
                 out.write("\n")
 
+    def batch_predict(self, input_data_list, outputs=None, **kwargs):
+        """Creates a batch centroid for a list of inputs using the local
+        cluster model. Allows to define some output settings to
+        decide the fields to be added to the input_data (centroid_name,
+        distance, etc.) and the name that we want to assign to these new
+        fields. The outputs argument accepts a dictionary with keys
+        "output_fields", to contain a list of the prediction properties to add
+        (["centroid_name", "distance"] by default) and "output_headers", to
+        contain a list of the headers to be used when adding them (identical
+        to "output_fields" list, by default).
+
+        :param input_data_list: List of input data to be predicted
+        :type input_data_list: list or Panda's dataframe
+        :param dict outputs: properties that define the headers and fields to
+                             be added to the input data
+        :return: the list of input data plus the predicted values
+        :rtype: list or Panda's dataframe depending on the input type in
+                input_data_list
+
+        """
+        if outputs is None:
+            outputs = {}
+        if outputs.get(OUT_NEW_FIELDS) is None:
+            new_fields = DFT_OUTPUTS
+        if outputs.get(OUT_NEW_HEADERS):
+            new_headers = outputs.get(OUT_NEW_HEADERS, new_fields)
+            if len(new_fields) > len(new_headers):
+                new_headers.expand(new_fields[len(new_headers):])
+            else:
+                new_headers = new_headers[0: len(new_fields)]
+        else:
+            new_headers = new_fields
+        dataframe = False
+        if PANDAS_READY and isinstance(input_data_list, DataFrame):
+            dataframe = True
+            inner_data_list = input_data_list.to_dict('records')
+        else:
+            inner_data_list = input_data_list[:]
+        for input_data in inner_data_list:
+            prediction = self.centroid(input_data, **kwargs)
+            for index, key in enumerate(new_fields):
+                input_data[new_headers[index]] = prediction[key]
+        if PANDAS_READY and dataframe:
+            return DataFrame.from_dict(inner_data_list)
+        return inner_data_list
+
     def dump(self, output=None, cache_set=None):
         """Uses msgpack to serialize the resource object
         If cache_set is filled with a cache set method, the method is called
