@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+#pylint: disable=wrong-import-position,ungrouped-imports
 #
-# Copyright 2017-2021 BigML
+# Copyright 2017-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +21,7 @@ This module defines a Deepnet to make predictions locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your models
 offline.
 
@@ -40,9 +41,14 @@ deepnet.predict({"petal length": 3, "petal width": 1})
 
 """
 import logging
+import os
 
 from functools import cmp_to_key
 
+# avoiding tensorflow info logging
+os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
+logging.getLogger('tensorflow').setLevel(logging.ERROR)
+
 from bigml.api import FINISHED
 from bigml.api import get_status, get_api_connection, get_deepnet_id
 from bigml.util import cast, use_cache, load, PRECISION
@@ -50,15 +56,33 @@ from bigml.basemodel import get_resource_dict, extract_objective
 from bigml.modelfields import ModelFields
 from bigml.laminar.constants import NUMERIC
 from bigml.model import parse_operating_point, sort_categories
+from bigml.constants import REGIONS, REGIONS_OPERATION_SETTINGS, \
+    DEFAULT_OPERATION_SETTINGS, REGION_SCORE_ALIAS, REGION_SCORE_THRESHOLD, \
+    IMAGE
 
 import bigml.laminar.numpy_ops as net
 import bigml.laminar.preprocess_np as pp
 
+try:
+    import tensorflow as tf
+    tf.autograph.set_verbosity(0)
+    LAMINAR_VERSION = False
+except ModuleNotFoundError:
+    LAMINAR_VERSION = True
+
+try:
+    from sensenet.models.wrappers import create_model
+    from bigml.images.utils import to_relative_coordinates
+    from bigml.constants import IOU_REMOTE_SETTINGS
+except ModuleNotFoundError:
+    LAMINAR_VERSION = True
+
 LOGGER = logging.getLogger('BigML')
 
 MEAN = "mean"
 STANDARD_DEVIATION = "stdev"
 
+
 def moments(amap):
     """Extracts mean and stdev
 
@@ -85,20 +109,34 @@ class Deepnet(ModelFields):
 
     """
 
-    def __init__(self, deepnet, api=None, cache_get=None):
+    def __init__(self, deepnet, api=None, cache_get=None,
+                 operation_settings=None):
         """The Deepnet constructor can be given as first argument:
             - a deepnet structure
             - a deepnet id
             - a path to a JSON file containing a deepnet structure
 
+        :param deepnet: The deepnet info or reference
+        :param api: Connection object that will be used to download the deepnet
+                    info if not locally available
+        :param cache_get: Get function that handles memory-cached objects
+        :param operation_settings: Dict object that contains operating options
+
+        The operation_settings will depend on the type of ML problem:
+         - regressions: no operation_settings allowed
+         - classifications: operating_point, operating_kind
+         - regions: bounding_box_threshold, iou_threshold and max_objects
         """
 
         if use_cache(cache_get):
             # using a cache to store the model attributes
             self.__dict__ = load(get_deepnet_id(deepnet), cache_get)
+            self.operation_settings = self._add_operation_settings(
+                operation_settings)
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.regression = False
         self.network = None
         self.networks = None
@@ -114,15 +152,16 @@ class Deepnet(ModelFields):
 
         if 'object' in deepnet and isinstance(deepnet['object'], dict):
             deepnet = deepnet['object']
+            self.dataset_id = deepnet.get('dataset')
         self.input_fields = deepnet['input_fields']
         self.default_numeric_value = deepnet.get('default_numeric_value')
         if 'deepnet' in deepnet and isinstance(deepnet['deepnet'], dict):
             status = get_status(deepnet)
             objective_field = deepnet['objective_fields']
-            deepnet = deepnet['deepnet']
+            deepnet_info = deepnet['deepnet']
             if 'code' in status and status['code'] == FINISHED:
-                self.fields = deepnet['fields']
-                missing_tokens = deepnet.get('missing_tokens')
+                self.fields = deepnet_info['fields']
+                missing_tokens = deepnet_info.get('missing_tokens')
                 ModelFields.__init__(
                     self, self.fields,
                     objective_id=extract_objective(objective_field),
@@ -130,19 +169,54 @@ class Deepnet(ModelFields):
 
                 self.regression = \
                     self.fields[self.objective_id]['optype'] == NUMERIC
-                if not self.regression:
+                self.regions = \
+                    self.fields[self.objective_id]['optype'] == REGIONS
+                if not self.regression and not self.regions:
                     # order matters
                     self.objective_categories = self.categories[
                         self.objective_id]
                     self.class_names = sorted(self.objective_categories)
 
-                self.missing_numerics = deepnet.get('missing_numerics', False)
-                if 'network' in deepnet:
-                    network = deepnet['network']
+                self.missing_numerics = deepnet_info.get('missing_numerics',
+                                                         False)
+                self.operation_settings = self._add_operation_settings(
+                    operation_settings)
+                if 'network' in deepnet_info:
+                    network = deepnet_info['network']
                     self.network = network
                     self.networks = network.get('networks', [])
+                    # old deepnets might use the latter option
+                    if self.networks:
+                        self.output_exposition = self.networks[0].get(
+                            "output_exposition")
+                    else:
+                        self.output_exposition = None
+                    self.output_exposition = self.network.get(
+                        "output_exposition", self.output_exposition)
                     self.preprocess = network.get('preprocess')
                     self.optimizer = network.get('optimizer', {})
+                if LAMINAR_VERSION:
+                    if self.regions:
+                        raise ValueError("Failed to find the extra libraries"
+                                         " that are compulsory for predicting "
+                                         "regions. Please, install them by "
+                                         "running \n"
+                                         "pip install bigml[images]")
+                    self.deepnet = None
+                    for _, field in self.fields.items():
+                        if field["optype"] == IMAGE:
+                            raise ValueError("This deepnet cannot be predicted"
+                                             " as some required libraries are "
+                                             "not available for this OS.")
+                else:
+                    if self.regions:
+                        settings = self.operation_settings or {}
+                        settings.update(IOU_REMOTE_SETTINGS)
+                    else:
+                        settings = None
+                    self.deepnet = create_model(deepnet,
+                        settings=settings)
+
             else:
                 raise Exception("The deepnet isn't finished yet")
         else:
@@ -150,6 +224,25 @@ class Deepnet(ModelFields):
                             " find the 'deepnet' key in the resource:\n\n%s" %
                             deepnet)
 
+    def _add_operation_settings(self, operation_settings):
+        """Checks and adds the user-given operation settings """
+        if operation_settings is None:
+            return None
+        if self.regression:
+            raise ValueError("No operating settings are allowed"
+                             " for regressions")
+        allowed_settings = REGIONS_OPERATION_SETTINGS if \
+            self.regions else DEFAULT_OPERATION_SETTINGS
+        settings = {setting: operation_settings[setting] for
+            setting in operation_settings.keys() if setting in
+            allowed_settings
+        }
+        if REGION_SCORE_ALIAS in settings:
+            settings[REGION_SCORE_THRESHOLD] = settings[
+                REGION_SCORE_ALIAS]
+            del settings[REGION_SCORE_ALIAS]
+        return settings
+
     def fill_array(self, input_data, unique_terms):
         """ Filling the input array for the network with the data in the
         input_data dictionary. Numeric missings are added as a new field
@@ -173,7 +266,10 @@ class Deepnet(ModelFields):
                 category = unique_terms.get(field_id)
                 if category is not None:
                     category = category[0][0]
-                columns.append([category])
+                if LAMINAR_VERSION:
+                    columns.append([category])
+                else:
+                    columns.append(category)
             else:
                 # when missing_numerics is True and the field had missings
                 # in the training data, then we add a new "is missing?" element
@@ -181,14 +277,16 @@ class Deepnet(ModelFields):
                 # missing or not in the input data
                 if self.missing_numerics \
                         and self.fields[field_id][\
-                        "summary"]["missing_count"] > 0:
+                        "summary"].get("missing_count", 0) > 0:
                     if field_id in input_data:
                         columns.extend([input_data[field_id], 0.0])
                     else:
                         columns.extend([0.0, 1.0])
                 else:
                     columns.append(input_data.get(field_id))
-        return pp.preprocess(columns, self.preprocess)
+        if LAMINAR_VERSION:
+            return pp.preprocess(columns, self.preprocess)
+        return columns
 
     def predict(self, input_data, operating_point=None, operating_kind=None,
                 full=False):
@@ -222,6 +320,15 @@ class Deepnet(ModelFields):
 
         # Checks and cleans input_data leaving the fields used in the model
         unused_fields = []
+
+        if self.regions:
+            # Only a single image file is allowed as input.
+            # Sensenet predictions are using absolute coordinates, so we need
+            # to change it to relative and set the decimal precision
+            prediction = to_relative_coordinates(input_data,
+                                                 self.deepnet(input_data))
+            return {"prediction": prediction}
+
         norm_input_data = self.filter_input_data( \
             input_data, add_unused_fields=full)
         if full:
@@ -233,6 +340,11 @@ class Deepnet(ModelFields):
         # When operating_point is used, we need the probabilities
         # of all possible classes to decide, so se use
         # the `predict_probability` method
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+        if operating_kind is None and self.operation_settings is not None:
+            operating_kind = self.operation_settings.get("operating_kind")
+
         if operating_point:
             if self.regression:
                 raise ValueError("The operating_point argument can only be"
@@ -248,13 +360,18 @@ class Deepnet(ModelFields):
 
         # Computes text and categorical field expansion
         unique_terms = self.get_unique_terms(norm_input_data)
-
         input_array = self.fill_array(norm_input_data, unique_terms)
-
-        if self.networks:
-            prediction = self.predict_list(input_array)
+        if self.deepnet is not None:
+            prediction = list(self.deepnet(input_array)[0])
+            # prediction is now a numpy array of probabilities for classification
+            # and a numpy array with the value for regressions
+            prediction = self.to_prediction(prediction)
         else:
-            prediction = self.predict_single(input_array)
+            # no tensorflow
+            if self.networks:
+                prediction = self.predict_list(input_array)
+            else:
+                prediction = self.predict_single(input_array)
         if full:
             if not isinstance(prediction, dict):
                 prediction = {"prediction": prediction}
@@ -267,7 +384,6 @@ class Deepnet(ModelFields):
 
     def predict_single(self, input_array):
         """Makes a prediction with a single network
-
         """
         if self.network['trees'] is not None:
             input_array = pp.tree_transform(input_array, self.network['trees'])
@@ -277,7 +393,6 @@ class Deepnet(ModelFields):
 
     def predict_list(self, input_array):
         """Makes predictions with a list of networks
-
         """
         if self.network['trees'] is not None:
             input_array_trees = pp.tree_transform(input_array,
@@ -296,11 +411,10 @@ class Deepnet(ModelFields):
         """Prediction with one model
 
         """
-
         layers = net.init_layers(model['layers'])
         y_out = net.propagate(input_array, layers)
         if self.regression:
-            y_mean, y_stdev = moments(model['output_exposition'])
+            y_mean, y_stdev = moments(self.output_exposition)
             y_out = net.destandardize(y_out, y_mean, y_stdev)
             return y_out[0][0]
 
@@ -311,12 +425,16 @@ class Deepnet(ModelFields):
 
         """
         if self.regression:
+            if not LAMINAR_VERSION:
+                y_out = y_out[0]
             return float(y_out)
-        prediction = sorted(enumerate(y_out[0]), key=lambda x: -x[1])[0]
+        if LAMINAR_VERSION:
+            y_out = y_out[0]
+        prediction = sorted(enumerate(y_out), key=lambda x: -x[1])[0]
         prediction = {"prediction": self.class_names[prediction[0]],
                       "probability": round(prediction[1], PRECISION),
                       "distribution": [{"category": category,
-                                        "probability": round(y_out[0][i],
+                                        "probability": round(y_out[i],
                                                              PRECISION)} \
             for i, category in enumerate(self.class_names)]}
 
@@ -325,7 +443,8 @@ class Deepnet(ModelFields):
     def predict_probability(self, input_data, compact=False):
         """Predicts a probability for each possible output class,
         based on input values.  The input fields must be a dictionary
-        keyed by field name or field ID.
+        keyed by field name or field ID. This method is not available for
+        regions objectives
 
         :param input_data: Input data to be predicted
         :param compact: If False, prediction is returned as a list of maps, one
@@ -334,6 +453,9 @@ class Deepnet(ModelFields):
                         respectively.  If True, returns a list of probabilities
                         ordered by the sorted order of the class names.
         """
+        if self.regions:
+            raise ValueError("The .predict_probability method cannot be used"
+                             " to predict regions.")
         if self.regression:
             prediction = self.predict(input_data, full=not compact)
             if compact:
@@ -346,6 +468,7 @@ class Deepnet(ModelFields):
             return [category['probability'] for category in distribution]
         return distribution
 
+    #pylint: disable=locally-disabled,invalid-name
     def _sort_predictions(self, a, b, criteria):
         """Sorts the categories in the predicted node according to the
         given criteria
@@ -380,7 +503,8 @@ class Deepnet(ModelFields):
         """
 
         kind, threshold, positive_class = parse_operating_point( \
-            operating_point, ["probability"], self.class_names)
+            operating_point, ["probability"], self.class_names,
+            self.operation_settings)
         predictions = self.predict_probability(input_data, False)
         position = self.class_names.index(positive_class)
         if predictions[position][kind] > threshold:
