@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,7 +19,7 @@
 This module defines an Ensemble to make predictions locally using its
 associated models.
 
-This module can not only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your models
 offline.
 
@@ -44,6 +44,7 @@ import json
 import os
 
 from functools import cmp_to_key
+from copy import deepcopy
 
 from bigml.exceptions import NoRootDecisionTree
 from bigml.api import get_ensemble_id, get_model_id, get_api_connection
@@ -83,22 +84,28 @@ class Ensemble(ModelFields):
        that can be used to generate predictions locally.
        The expected arguments are:
 
-       ensemble: ensemble object or id, list of model objects or
-                 ids or list of local model objects (see Model)
-       api: connection object. If None, a new connection object is
-            instantiated.
-       max_models: integer that limits the number of models instantiated and
-                   held in memory at the same time while predicting. If None,
-                   no limit is set and all the ensemble models are
-                   instantiated and held in memory permanently.
-       cache_get: user-provided function that should return the JSON
-                  information describing the model or the corresponding
-                  Model object. Can be used to read these objects from a
-                  cache storage.
     """
 
-    def __init__(self, ensemble, api=None, max_models=None, cache_get=None):
+    #pylint: disable=locally-disabled,broad-except,access-member-before-definition
+    def __init__(self, ensemble, api=None, max_models=None, cache_get=None,
+                 operation_settings=None):
+        """
+        :param ensemble: ensemble object or id, list of ensemble model
+                        objects or ids or list of ensemble obj and local model
+                        objects (see Model)
+        :param api: connection object. If None, a new connection object is
+                    instantiated.
+        :param max_models: integer that limits the number of models instantiated
+                           and held in memory at the same time while predicting.
+                           If None, no limit is set and all the ensemble models
+                           are instantiated and held in memory permanently.
+        :param cache_get: user-provided function that should return the JSON
+                          information describing the model or the corresponding
+                          Ensemble object. Can be used to read these objects
+                          from a cache storage.
+        :param operation_settings: Dict object that contains operating options
 
+        """
         self.model_splits = []
         self.multi_model = None
         self.api = get_api_connection(api)
@@ -109,24 +116,30 @@ class Ensemble(ModelFields):
             # using a cache to store the model attributes
             self.__dict__ = load(get_ensemble_id(ensemble), cache_get)
             self.api = get_api_connection(api)
+            self.operation_settings = self._add_operation_settings(
+                operation_settings)
             if len(self.models_splits) == 1:
                 # retrieve the models from a cache get function
                 try:
-                    models = [Model(model_id, cache_get=cache_get)
+                    models = [Model(model_id, cache_get=cache_get,
+                                    operation_settings=operation_settings)
                               for model_id
                               in self.models_splits[0]]
                 except Exception as exc:
                     raise Exception('Error while calling the user-given'
                                     ' function %s: %s' %
                                     (cache_get.__name__, str(exc)))
-                self.multi_model = MultiModel(models,
-                                              self.api,
-                                              fields=self.fields,
-                                              class_names=self.class_names,
-                                              cache_get=cache_get)
+                self.multi_model = MultiModel(
+                    models,
+                    self.api,
+                    fields=self.fields,
+                    class_names=self.class_names,
+                    cache_get=cache_get,
+                    operation_settings=operation_settings)
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.objective_id = None
         self.distributions = None
         self.distribution = None
@@ -138,34 +151,75 @@ class Ensemble(ModelFields):
         query_string = ONLY_MODEL
         no_check_fields = False
         self.input_fields = []
-        if isinstance(ensemble, list):
-            if all([isinstance(model, Model) for model in ensemble]):
-                models = ensemble
-                self.model_ids = [local_model.resource_id for local_model in
-                                  models]
-            else:
-                try:
-                    models = [get_model_id(model) for model in ensemble]
-                    self.model_ids = models
-                except ValueError as exc:
-                    raise ValueError('Failed to verify the list of models.'
-                                     ' Check your model id values: %s' %
-                                     str(exc))
+        child_api = self.api
 
-        else:
+
+        models = []
+        if isinstance(ensemble, list):
+            try:
+                if isinstance(ensemble[0], dict) and \
+                        get_ensemble_id(ensemble[0]):
+                    number_of_models = len(ensemble) - 1
+                    model_list = ensemble
+                    ensemble = model_list[0]
+                    if len(ensemble["object"]["models"]) == number_of_models:
+                        model_list = model_list[1:]
+                    else:
+                        raise ValueError("The provided list of models does not"
+                                         " match the ensemble list of models.")
+                    try:
+                        models = [Model(
+                            model, operation_settings=operation_settings)
+                            for model in model_list]
+                    except Exception:
+                        models = model_list
+                else:
+                    # only list of models (old ensembles)
+                    models = ensemble
+                    ensemble=None
+            except ValueError:
+                # only list of models (old ensembles)
+                models = ensemble
+                ensemble = None
+            if models:
+                if all(isinstance(model, Model) for model in models):
+                    self.model_ids = [local_model.resource_id for local_model in
+                                      models]
+                else:
+                    try:
+                        models = [get_model_id(model) for model in models]
+                        self.model_ids = models
+                    except ValueError as exc:
+                        raise ValueError('Failed to verify the list of models.'
+                                         ' Check your model id values: %s' %
+                                         str(exc))
+        if ensemble:
             ensemble = self.get_ensemble_resource(ensemble)
             self.resource_id = get_ensemble_id(ensemble)
+            shared_ref = self.resource_id.replace("shared/", "") if \
+                self.resource_id.startswith("shared/") else None
+            if shared_ref is not None:
+                child_api = deepcopy(self.api)
+                child_api.shared_ref = shared_ref
+            elif hasattr(self.api, "shared_ref") and \
+                    self.api.shared_ref is not None:
+                child_api = deepcopy(self.api)
+                # adding the resource ID to the sharing chain
+                child_api.shared_ref += ",%s" % self.resource_id
+
             if not check_local_but_fields(ensemble):
                 # avoid checking fields because of old ensembles
                 ensemble = retrieve_resource(self.api, self.resource_id,
                                              no_check_fields=True)
 
+            self.dataset_id = ensemble.get('object', {}).get('dataset')
             if ensemble['object'].get('type') == BOOSTING:
                 self.boosting = ensemble['object'].get('boosting')
-            models = ensemble['object']['models']
             self.distributions = ensemble['object'].get('distributions', [])
             self.importance = ensemble['object'].get('importance', [])
-            self.model_ids = models
+            self.model_ids = ensemble['object']['models']
+            if not models:
+                models = self.model_ids
             # new ensembles have the fields structure
             if ensemble['object'].get('ensemble'):
                 self.fields = ensemble['object'].get( \
@@ -187,7 +241,8 @@ class Ensemble(ModelFields):
                 if use_cache(cache_get):
                     # retrieve the models from a cache get function
                     try:
-                        models = [Model(model_id, cache_get=cache_get)
+                        models = [Model(model_id, cache_get=cache_get,
+                            operation_settings=operation_settings)
                                   for model_id
                                   in self.models_splits[0]]
                         self.cache_get = cache_get
@@ -197,7 +252,7 @@ class Ensemble(ModelFields):
                                         (cache_get.__name__, str(exc)))
                 else:
                     models = [retrieve_resource( \
-                        self.api,
+                        child_api,
                         model_id,
                         query_string=query_string,
                         no_check_fields=no_check_fields)
@@ -212,7 +267,8 @@ class Ensemble(ModelFields):
                     # retrieve the models from a cache get function
                     try:
                         model = Model(self.models_splits[0][0],
-                                      cache_get=cache_get)
+                                      cache_get=cache_get,
+                                      operation_settings=operation_settings)
                         self.cache_get = cache_get
                     except Exception as exc:
                         raise Exception('Error while calling the user-given'
@@ -220,7 +276,7 @@ class Ensemble(ModelFields):
                                         (cache_get.__name__, str(exc)))
                 else:
                     model = retrieve_resource( \
-                        self.api,
+                        child_api,
                         self.models_splits[0][0],
                         query_string=query_string,
                         no_check_fields=no_check_fields)
@@ -275,10 +331,12 @@ class Ensemble(ModelFields):
             objective_id=self.objective_id)
 
         if len(self.models_splits) == 1:
-            self.multi_model = MultiModel(models,
-                                          self.api,
-                                          fields=self.fields,
-                                          class_names=self.class_names)
+            self.multi_model = MultiModel(
+                models,
+                self.api,
+                fields=self.fields,
+                class_names=self.class_names,
+                operation_settings=operation_settings)
             for index, model in enumerate(self.multi_model.models):
                 self.multi_model.models[index].term_forms = self.term_forms
 
@@ -547,6 +605,7 @@ class Ensemble(ModelFields):
 
         return models
 
+    #pylint: disable=locally-disabled,invalid-name
     def _sort_predictions(self, a, b, criteria):
         """Sorts the categories in the predicted node according to the
         given criteria
@@ -563,7 +622,8 @@ class Ensemble(ModelFields):
 
         """
         kind, threshold, positive_class = parse_operating_point( \
-            operating_point, OPERATING_POINT_KINDS, self.class_names)
+            operating_point, OPERATING_POINT_KINDS,
+            self.class_names, self.operation_settings)
 
         try:
             predict_method = None
@@ -634,6 +694,7 @@ class Ensemble(ModelFields):
             del prediction["category"]
         return prediction
 
+    #pylint: disable=locally-disabled,protected-access
     def predict(self, input_data, method=None,
                 options=None, missing_strategy=LAST_PREDICTION,
                 operating_point=None, operating_kind=None, median=False,
@@ -709,6 +770,10 @@ class Ensemble(ModelFields):
         if median and method is None:
             # predictions with median are only available with old combiners
             method = PLURALITY_CODE
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+        if operating_kind is None and self.operation_settings is not None:
+            operating_kind = self.operation_settings.get("operating_kind")
 
         if method is None and operating_point is None and \
             operating_kind is None and not median:
