@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2021 BigML
+# Copyright 2017-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -356,7 +356,8 @@ class EnsemblePredictor():
         """
         if not os.path.isfile(directory) and not os.path.exists(directory):
             os.makedirs(directory)
-            open(os.path.join(directory, "__init__.py"), "w").close()
+            with open(os.path.join(directory, "__init__.py"), mode='w'):
+                pass
         for model_id in self.model_ids:
             local_model = Model(model_id, api=self.api,
                                 fields=self.fields)
