@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2021 BigML
+# Copyright 2021-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,9 +22,12 @@ class ResourceException(Exception):
     """Base class to any exception that arises from a bad structured resource
 
     """
-    pass
 
 
 class NoRootDecisionTree(ResourceException):
     """The decision tree structure has no "root" attribute """
-    pass
+
+
+
+class FaultyResourceError(Exception):
+    """Exception to be raised when retrieving a Faulty resource """
