@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+#pylint: disable=unbalanced-tuple-unpacking
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -48,13 +49,14 @@ import random
 
 from bigml.util import invert_dictionary, python_map_type, find_locale
 from bigml.util import DEFAULT_LOCALE
-from bigml.api import get_resource_type, get_fields
+from bigml.api_handlers.resourcehandler import get_resource_type, get_fields
 from bigml.constants import (
     SOURCE_PATH, DATASET_PATH, SUPERVISED_PATHS, FUSION_PATH,
-    RESOURCES_WITH_FIELDS, DEFAULT_MISSING_TOKENS)
+    RESOURCES_WITH_FIELDS, DEFAULT_MISSING_TOKENS, REGIONS)
 from bigml.io import UnicodeReader, UnicodeWriter
 
 LIST_LIMIT = 10
+REGIONS_ATTR = "labels"
 SUMMARY_HEADERS = ["field column", "field ID", "field name", "field label",
                    "field description", "field type", "preferred",
                    "missing count", "errors", "contents summary",
@@ -119,8 +121,17 @@ def attribute_summary(attribute_value, item_type, limit=None):
     """
     if attribute_value is None:
         return None
-    items = ["%s (%s)" % (item, instances) for
-             item, instances in attribute_value]
+    if item_type != REGIONS_ATTR:
+        items = ["%s (%s)" % (item, instances) for
+                 item, instances in attribute_value]
+        items_length = len(items)
+        if limit is None or limit > items_length:
+            limit = items_length
+        return "%s %s: %s" % (items_length, type_singular(item_type,
+                                                          items_length == 1),
+                              ", ".join(items[0: limit]))
+    items = ["%s (%s)" % (attr.get("label"), attr.get("count")) for
+             attr in attribute_value]
     items_length = len(items)
     if limit is None or limit > items_length:
         limit = items_length
@@ -128,6 +139,7 @@ def attribute_summary(attribute_value, item_type, limit=None):
                                                       items_length == 1),
                           ", ".join(items[0: limit]))
 
+
 def type_singular(item_type, singular=False):
     """Singularizes item types if needed
 
@@ -137,6 +149,50 @@ def type_singular(item_type, singular=False):
     return item_type
 
 
+def numeric_example(numeric_summary):
+    """Generates a random numeric example in the gaussian defined by
+    mean and sigma in the numeric_summary
+
+    """
+    try:
+        mean = numeric_summary.get("mean")
+        sigma = numeric_summary.get("standard_deviation")
+        minimum = numeric_summary.get("minimum")
+        maximum = numeric_summary.get("maximum")
+        value = -1
+        while value < minimum or value > maximum:
+            value = random.gauss(mean, sigma)
+        return value
+    except TypeError:
+        return None
+
+
+def sorted_headers(fields):
+    """Listing the names of the fields as ordered in the original dataset.
+    The `fields` parameter is a Fields object.
+    """
+    header_names = []
+    header_ids = []
+    for column in fields.fields_columns:
+        header_names.append(fields.fields[
+            fields.fields_by_column_number[column]]["name"])
+        header_ids.append(fields.fields_by_column_number[column])
+
+    return header_names, header_ids
+
+
+def get_new_fields(output_fields):
+    """Extracts the sexpr and names of the output fields in a dataset
+    generated from a new_fields transformation.
+    """
+    new_fields = []
+    for output_field in output_fields:
+        sexp = output_field.get("generator")
+        names = output_field.get("names")
+        new_fields.append({"field": sexp, "names": names})
+    return new_fields
+
+
 class Fields():
     """A class to deal with BigML auto-generated ids.
 
@@ -196,7 +252,10 @@ class Fields():
         if objective_field is None and objective_column is not None:
             objective_field = objective_column
             objective_field_present = True
-        self.update_objective_field(objective_field, objective_field_present)
+        if self.fields:
+            # empty composite sources will not have an objective field
+            self.update_objective_field(objective_field,
+                                        objective_field_present)
 
     def update_objective_field(self, objective_field, objective_field_present,
                                headers=None):
@@ -474,6 +533,11 @@ class Fields():
                     field_summary.append( \
                         attribute_summary(categories, "categorìes",
                                           limit=LIST_LIMIT))
+                elif field['optype'] == REGIONS:
+                    labels_info = field_summary_value.get("labels")
+                    field_summary.append( \
+                        attribute_summary(labels_info, "labels",
+                                          limit=LIST_LIMIT))
                 elif field['optype'] == "text":
                     terms = field_summary_value.get("tag_cloud")
                     field_summary.append( \
@@ -498,6 +562,7 @@ class Fields():
         if writer is None:
             return summary
         writer.close_writer()
+        return filename
 
     def new_fields_structure(self, csv_attributes_file=None,
                              attributes=None, out_file=None):
@@ -515,7 +580,7 @@ class Fields():
         """
         if csv_attributes_file is not None:
             reader = UnicodeReader(csv_attributes_file).open_reader()
-            attributes = [row for row in reader]
+            attributes = list(reader)
         new_fields_structure = {}
         if "field ID" in attributes[0] or "field column" in attributes[0]:
             # headers are used
@@ -532,8 +597,10 @@ class Fields():
                     try:
                         field_column = int(new_attributes.get("field column"))
                     except TypeError:
-                        raise ValueError("Field column %s not found"
-                                         " in this resource" % field_column)
+                        raise ValueError(
+                            "Field column %s not found"
+                            " in this resource" % new_attributes.get(
+                            "field_column"))
                     if not field_column in self.fields_columns:
                         raise ValueError("Field column %s not found"
                                          " in this resource" % field_column)
@@ -589,6 +656,7 @@ class Fields():
             raise IOError("Failed writing the fields structure file in"
                           " %s- Please, check your arguments." %
                           out_file)
+        return out_file
 
     def training_data_example(self, missings=False):
         """Generates an example of training data based on the contents of the
@@ -606,12 +674,7 @@ class Fields():
                     if missings and random.randint(0, 5) > 3:
                         value = None
                     else:
-                        try:
-                            mean = field["summary"]["mean"]
-                            sigma = field["summary"]["standard_deviation"]
-                            value = random.gauss(mean, sigma)
-                        except TypeError:
-                            value = None
+                        value = numeric_example(field["summary"])
                 if optype == "categorical":
                     if missings and random.randint(0, 5) > 3:
                         value = None
@@ -633,6 +696,26 @@ class Fields():
                         items_number = len(field["summary"]["items"])
                         index = random.randint(0, items_number - 1)
                         value = field["summary"]["items"][index][0]
+                if optype == REGIONS:
+                    if missings and random.randint(0, 5) > 3:
+                        value = None
+                    else:
+                        labels_number = len(field["summary"]["labels"])
+                        index = random.randint(0, labels_number - 1)
+                        field_summary = field["summary"]["labels"][index]
+                        label = field_summary["label"]
+                        xmin = numeric_example(field_summary["xmin"])
+                        xmax = numeric_example(field_summary["xmax"])
+                        ymin = numeric_example(field_summary["ymin"])
+                        ymax = numeric_example(field_summary["ymax"])
+                        #pylint: disable=locally-disabled,too-many-boolean-expressions
+                        if None in [xmin, xmax, ymin, ymax] or xmax < xmin or \
+                                ymax < ymin or xmin < 0 or xmax < 0 or \
+                                ymin < 0 or ymax < 0:
+                            value = []
+                        else:
+                            value = [[label, xmin, xmax, ymin, ymax]]
+
                 if value is not None:
                     training_data.update({field["name"]: value})
         return training_data
