@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,7 +19,7 @@
 This module defines a Fusion to make predictions locally using its
 associated models.
 
-This module can not only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your models
 offline.
 
@@ -39,16 +39,15 @@ fusion.predict({"petal length": 3, "petal width": 1})
 
 """
 import logging
-import json
-import os
 
 from functools import cmp_to_key
+from copy import deepcopy
 
-from bigml.api import BigML, get_fusion_id, get_resource_type, \
+from bigml.api import get_fusion_id, get_resource_type, \
     get_api_connection
 from bigml.model import parse_operating_point, sort_categories
 from bigml.model import LAST_PREDICTION
-from bigml.basemodel import get_resource_dict, retrieve_resource
+from bigml.basemodel import get_resource_dict
 from bigml.multivotelist import MultiVoteList
 from bigml.util import cast, check_no_missing_numerics, use_cache, load, \
     dump, dumps, NUMERIC
@@ -129,12 +128,14 @@ class Fusion(ModelFields):
                   cache storage.
     """
 
-    def __init__(self, fusion, api=None, max_models=None, cache_get=None):
+    def __init__(self, fusion, api=None, max_models=None, cache_get=None,
+                 operation_settings=None):
 
         if use_cache(cache_get):
             # using a cache to store the model attributes
             self.__dict__ = load(get_fusion_id(fusion), cache_get)
             self.api = get_api_connection(api)
+            self.operation_settings = operation_settings
             return
 
         self.resource_id = None
@@ -150,7 +151,8 @@ class Fusion(ModelFields):
         self.api = get_api_connection(api)
 
         self.resource_id, fusion = get_resource_dict( \
-            fusion, "fusion", api=self.api)
+            fusion,
+            "fusion", api=self.api)
 
         if 'object' in fusion:
             fusion = fusion.get('object', {})
@@ -174,12 +176,24 @@ class Fusion(ModelFields):
 
         # Downloading the model information to cache it
         if self.api.storage is not None or cache_get is not None:
+            # adding shared_ref to the API info when donwloading children
+            api = self.api
+            if self.resource_id.startswith("shared"):
+                api = deepcopy(api)
+                api.shared_ref = self.resource_id.replace("shared/", "")
+            elif hasattr(api, "shared_ref") and \
+                    api.shared_ref is not None:
+                api = deepcopy(api)
+                # adding the resource ID to the sharing chain
+                api.shared_ref += ",%s" % self.resource_id
             for model_id in self.model_ids:
                 if get_resource_type(model_id) == "fusion":
-                    Fusion(model_id, api=self.api, cache_get=cache_get)
+                    Fusion(model_id, api=api, cache_get=cache_get,
+                           operation_settings=operation_settings)
                 else:
-                    SupervisedModel(model_id, api=self.api,
-                                    cache_get=cache_get)
+                    SupervisedModel(model_id, api=api,
+                                    cache_get=cache_get,
+                                    operation_settings=operation_settings)
 
         if max_models is None:
             self.models_splits = [self.model_ids]
@@ -211,43 +225,6 @@ class Fusion(ModelFields):
         self.regression = \
             self.fields[self.objective_id].get('optype') == NUMERIC
 
-
-    def get_fusion_resource(self, fusion):
-        """Extracts the fusion resource info. The fusion argument can be
-           - a path to a local file
-           - an fusion id
-        """
-        # the string can be a path to a JSON file
-        if isinstance(fusion, str):
-            try:
-                path = os.path.dirname(os.path.abspath(fusion))
-                with open(fusion) as fusion_file:
-                    fusion = json.load(fusion_file)
-                    self.resource_id = get_fusion_id(fusion)
-                    if self.resource_id is None:
-                        raise ValueError("The JSON file does not seem"
-                                         " to contain a valid BigML fusion"
-                                         " representation.")
-                    api = BigML(storage=path)
-            except IOError:
-                # if it is not a path, it can be an fusion id
-                self.resource_id = get_fusion_id(fusion)
-                if self.resource_id is None:
-                    if fusion.find('fusion/') > -1:
-                        raise Exception(
-                            api.error_message(fusion,
-                                              resource_type='fusion',
-                                              method='get'))
-                    raise IOError("Failed to open the expected JSON file"
-                                  " at %s" % fusion)
-            except ValueError:
-                raise ValueError("Failed to interpret %s."
-                                 " JSON file expected.")
-        if not isinstance(fusion, dict):
-            fusion = retrieve_resource(api, self.resource_id,
-                                       no_check_fields=False)
-        return fusion
-
     def list_models(self):
         """Lists all the model/ids that compound the fusion.
 
@@ -423,6 +400,9 @@ class Fusion(ModelFields):
         # When operating_point is used, we need the probabilities
         # of all possible classes to decide, so se use
         # the `predict_probability` method
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+
         if operating_point:
             if self.regression:
                 raise ValueError("The operating_point argument can only be"
@@ -455,10 +435,14 @@ class Fusion(ModelFields):
         """Computes the prediction based on a user-given operating point.
 
         """
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+
         # only probability is allowed as operating kind
         operating_point.update({"kind": "probability"})
         kind, threshold, positive_class = parse_operating_point( \
-            operating_point, OPERATING_POINT_KINDS, self.class_names)
+            operating_point, OPERATING_POINT_KINDS, self.class_names,
+            self.operation_settings)
         predictions = self.predict_probability(input_data,
                                                missing_strategy, False)
 
@@ -480,6 +464,7 @@ class Fusion(ModelFields):
         del prediction["category"]
         return prediction
 
+    #pylint: disable=locally-disabled,invalid-name
     def _sort_predictions(self, a, b, criteria):
         """Sorts the categories in the predicted node according to the
         given criteria
