@@ -40,3 +40,8 @@ MAX_VALIDATION_ROWS = 4096
 LEARN_INCREMENT = 8
 MAX_QUEUE = LEARN_INCREMENT * 4
 N_CANDIDATES = MAX_QUEUE * 64
+
+# Activation constants
+ALPHA = 1.6732632423543772848170429916717
+LAMBDA = 1.0507009873554804934193349852946
+LEAKY_RELU_CONST = 0.1
