@@ -1,3 +1,22 @@
+# -*- coding: utf-8 -*-
+#pylint: disable=invalid-name,missing-function-docstring
+#
+# Copyright 2017-2022 BigML
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
+"""Pre-processing fields for deepent computations """
+
 import math
 import numpy as np
 
@@ -37,6 +56,7 @@ def standardize(vector, mn, stdev):
     newvec = np.vectorize(fill_dft)(newvec)
     return newvec
 
+#pylint: disable=locally-disabled,unused-argument
 def binarize(vector, zero, one):
     if one == 0.0:
         vector[vector == one] = 1.0
@@ -103,8 +123,7 @@ def get_embedding(X, model):
             preds /= len(model)
 
         return preds
-    else:
-        raise ValueError("Model is unknown type!")
+    raise ValueError("Model is unknown type!")
 
 
 def tree_transform(X, trees):
