@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a Linear Regression to make predictions locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your linear
 regressions offline.
 
@@ -111,6 +111,7 @@ class LinearRegression(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.input_fields = []
         self.term_forms = {}
         self.tag_clouds = {}
@@ -134,6 +135,7 @@ class LinearRegression(ModelFields):
         if 'object' in linear_regression and \
             isinstance(linear_regression['object'], dict):
             linear_regression = linear_regression['object']
+            self.dataset_id = linear_regression.get('dataset')
         try:
             self.input_fields = linear_regression.get("input_fields", [])
             self.default_numeric_value = linear_regression.get( \
