@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a Logistic Regression to make predictions locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your logistic
 regressions offline.
 
@@ -86,17 +86,40 @@ class LogisticRegression(ModelFields):
     Uses a BigML remote logistic regression model to build a local version
     that can be used to generate predictions locally.
 
+
     """
 
-    def __init__(self, logistic_regression, api=None, cache_get=None):
+    #pylint: disable=locally-disabled,invalid-name
+    def __init__(self, logistic_regression, api=None, cache_get=None,
+                 operation_settings=None):
+        """
+       :param logistic_regression: logistic_regression object or id, list of
+                                   ensemble model objects or ids or list of
+                                   ensemble obj and local model objects
+                                   (see Model)
+       :param api: connection object. If None, a new connection object is
+                   instantiated.
+       :param max_models: integer that limits the number of models instantiated
+                          and held in memory at the same time while predicting.
+                          If None, no limit is set and all the ensemble models
+                          are instantiated and held in memory permanently.
+       :param cache_get: user-provided function that should return the JSON
+                         information describing the model or the corresponding
+                         LogisticRegression object. Can be used to read these
+                         objects from a cache storage.
+       :param operation_settings: Dict object that contains operating options
+        """
 
         if use_cache(cache_get):
             # using a cache to store the model attributes
             self.__dict__ = load(get_logistic_regression_id( \
                 logistic_regression), cache_get)
+            self.operation_settings = self._add_operation_settings(
+                operation_settings)
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.class_names = None
         self.input_fields = []
         self.term_forms = {}
@@ -117,6 +140,7 @@ class LogisticRegression(ModelFields):
         self.lr_normalize = None
         self.balance_fields = None
         self.regularization = None
+        self.flat_coefficients = None
         api = get_api_connection(api)
 
         old_coefficients = False
@@ -127,6 +151,7 @@ class LogisticRegression(ModelFields):
         if 'object' in logistic_regression and \
             isinstance(logistic_regression['object'], dict):
             logistic_regression = logistic_regression['object']
+            self.dataset_id = logistic_regression.get('dataset')
         try:
             self.input_fields = logistic_regression.get("input_fields", [])
             self.default_numeric_value = logistic_regression.get(
@@ -176,7 +201,8 @@ class LogisticRegression(ModelFields):
                 ModelFields.__init__(
                     self, fields,
                     objective_id=objective_id, categories=True,
-                    numerics=True, missing_tokens=missing_tokens)
+                    numerics=True, missing_tokens=missing_tokens,
+                    operation_settings=operation_settings)
                 self.field_codings = logistic_regression_info.get( \
                   'field_codings', {})
                 self.format_field_codings()
@@ -205,6 +231,7 @@ class LogisticRegression(ModelFields):
                             " in the resource:\n\n%s" %
                             logistic_regression)
 
+    #pylint: disable=locally-disabled,invalid-name
     def _sort_predictions(self, a, b, criteria):
         """Sorts the categories in the predicted node according to the
         given criteria
@@ -239,8 +266,9 @@ class LogisticRegression(ModelFields):
 
         """
 
-        kind, threshold, positive_class = parse_operating_point( \
-            operating_point, ["probability"], self.class_names)
+        kind, threshold, positive_class = parse_operating_point(
+            operating_point, ["probability"],
+            self.class_names, self.operation_settings)
         predictions = self.predict_probability(input_data, False)
         position = self.class_names.index(positive_class)
         if predictions[position][kind] > threshold:
@@ -280,6 +308,7 @@ class LogisticRegression(ModelFields):
         del prediction["category"]
         return prediction
 
+    #pylint: disable=locally-disabled,consider-using-dict-items
     def predict(self, input_data,
                 operating_point=None, operating_kind=None,
                 full=False):
@@ -328,6 +357,11 @@ class LogisticRegression(ModelFields):
         # When operating_point is used, we need the probabilities
         # of all possible classes to decide, so se use
         # the `predict_probability` method
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+        if operating_kind is None and self.operation_settings is not None:
+            operating_kind = self.operation_settings.get("operating_kind")
+
         if operating_point:
             return self.predict_operating( \
                 norm_input_data, operating_point=operating_point)
