@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2013-2021 BigML
+# Copyright 2013-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a Model to make predictions locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your models
 offline.
 
@@ -48,6 +48,7 @@ model.python()
 
 """
 import logging
+#pylint: disable=locally-disabled,unused-import
 import locale
 
 from functools import cmp_to_key
@@ -102,10 +103,13 @@ PYTHON_CONV = {
     "day-of-week": "lambda x: int(locale.atof(x))",
     "day-of-month": "lambda x: int(locale.atof(x))"}
 
+
+#pylint: disable=locally-disabled,eval-used
 PYTHON_FUNC = {numtype: eval(function)
                for numtype, function in PYTHON_CONV.items()}
 
 
+#pylint: disable=locally-disabled,invalid-name
 def init_structure(to):
     """Creates the empty structure to store predictions depending on the
     chosen format.
@@ -118,6 +122,7 @@ def init_structure(to):
         else []
 
 
+#pylint: disable=locally-disabled,unused-argument,invalid-name,redefined-builtin
 def cast_prediction(full_prediction, to=None,
                     confidence=False, probability=False,
                     path=False, distribution=False,
@@ -154,11 +159,10 @@ def cast_prediction(full_prediction, to=None,
     prediction_properties = [ \
         "prediction", "confidence", "probability", "path", "distribution",
         "count", "next", "d_min", "d_max", "median", "unused_fields"]
-    prediction = True
     result = init_structure(to)
     for prop in prediction_properties:
         value = full_prediction.get(prop)
-        if eval(prop):
+        if prop == prediction_properties[0] or eval(prop):
             if to is None:
                 # tuple
                 result = result + (value,)
@@ -170,6 +174,7 @@ def cast_prediction(full_prediction, to=None,
     return result
 
 
+#pylint: disable=locally-disabled,invalid-name,redefined-outer-name
 def sort_categories(a, b, categories_list):
     """Sorts a list of dictionaries with category keys according to their
     value and order in the categories_list. If not found, alphabetic order is
@@ -188,11 +193,18 @@ def sort_categories(a, b, categories_list):
     return 0
 
 
-def parse_operating_point(operating_point, operating_kinds, class_names):
+def parse_operating_point(operating_point, operating_kinds, class_names,
+    operation_settings):
     """Checks the operating point contents and extracts the three defined
     variables
 
     """
+    # When operating_point is used, we need the probabilities
+    # of all possible classes to decide, so se use
+    # the `predict_probability` method
+    if operating_point is None and operation_settings is not None:
+        operating_point = operation_settings.get("operating_point")
+
     if "kind" not in operating_point:
         raise ValueError("Failed to find the kind of operating point.")
     if operating_point["kind"] not in operating_kinds:
@@ -315,12 +327,23 @@ class Model(BaseModel):
 
     """
 
-    def __init__(self, model, api=None, fields=None, cache_get=None):
+    def __init__(self, model, api=None, fields=None, cache_get=None,
+                 operation_settings=None):
         """The Model constructor can be given as first argument:
             - a model structure
             - a model id
             - a path to a JSON file containing a model structure
 
+        :param model: The model info or reference
+        :param api: Connection object that will be used to download the deepnet
+                    info if not locally available
+        :param cache_get: Get function that handles memory-cached objects
+        :param operation_settings: Dict object that contains operating options
+
+        The operation_settings will depend on the type of ML problem:
+         - regressions: no operation_settings allowed
+         - classifications: operating_point, operating_kind
+
         """
 
         if use_cache(cache_get):
@@ -329,6 +352,7 @@ class Model(BaseModel):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.ids_map = {}
         self.terms = {}
         self.regression = False
@@ -342,6 +366,7 @@ class Model(BaseModel):
 
         if 'object' in model and isinstance(model['object'], dict):
             model = model['object']
+            self.dataset_id = model.get('dataset')
 
         if 'model' in model and isinstance(model['model'], dict):
             status = get_status(model)
@@ -354,7 +379,8 @@ class Model(BaseModel):
 
                 self.default_numeric_value = model.get('default_numeric_value')
                 self.input_fields = model["input_fields"]
-                BaseModel.__init__(self, model, api=api, fields=fields)
+                BaseModel.__init__(self, model, api=api, fields=fields,
+                    operation_settings=operation_settings)
 
                 try:
                     root = model['model']['root']
@@ -402,11 +428,11 @@ class Model(BaseModel):
 
                 if terms:
                     # only the terms used in the model are kept
-                    for field_id in terms:
+                    for field_id, field_terms in terms.items():
                         if self.tag_clouds.get(field_id):
-                            self.tag_clouds[field_id] = terms[field_id]
+                            self.tag_clouds[field_id] = field_terms
                         elif self.items.get(field_id):
-                            self.items[field_id] = terms[field_id]
+                            self.items[field_id] = field_terms
 
                 if self.boosting:
                     self.tree_type = BOOSTING
@@ -556,7 +582,8 @@ class Model(BaseModel):
         """
 
         kind, threshold, positive_class = parse_operating_point( \
-            operating_point, OPERATING_POINT_KINDS, self.class_names)
+            operating_point, OPERATING_POINT_KINDS, self.class_names,
+            self.operation_settings)
         if kind == "probability":
             predictions = self.predict_probability(input_data,
                                                    missing_strategy, False)
@@ -582,6 +609,7 @@ class Model(BaseModel):
         del prediction["category"]
         return prediction
 
+    #pylint: disable=locally-disabled,invalid-name,redefined-outer-name
     def _sort_predictions(self, a, b, criteria):
         """Sorts the categories in the predicted node according to the
         given criteria
@@ -676,6 +704,11 @@ class Model(BaseModel):
         # Strips affixes for numeric values and casts to the final field type
         cast(norm_input_data, self.fields)
 
+        if operating_point is None and self.operation_settings is not None:
+            operating_point = self.operation_settings.get("operating_point")
+        if operating_kind is None and self.operation_settings is not None:
+            operating_kind = self.operation_settings.get("operating_kind")
+
         full_prediction = self._predict( \
             norm_input_data, missing_strategy=missing_strategy,
             operating_point=operating_point, operating_kind=operating_kind,
@@ -683,7 +716,6 @@ class Model(BaseModel):
         if full:
             return dict((key, value) for key, value in \
                 full_prediction.items() if value is not None)
-
         return full_prediction['prediction']
 
     def _predict(self, input_data, missing_strategy=LAST_PREDICTION,
