@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2013-2021 BigML
+# Copyright 2013-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -24,24 +24,31 @@ is used for local predictions.
 """
 import logging
 import re
+import copy
+import os
 
-from bigml_chronos import chronos
 
 from bigml.util import invert_dictionary, dump, dumps, DEFAULT_LOCALE
 from bigml.constants import DEFAULT_MISSING_TOKENS, FIELDS_PARENT, \
-    ENSEMBLE_PATH
+    ENSEMBLE_PATH, DEFAULT_OPERATION_SETTINGS
 from bigml.api_handlers.resourcehandler import get_resource_type
 from bigml.predicate import TM_FULL_TERM, TM_ALL
+from bigml.featurizer import Featurizer
 
+# avoiding tensorflow info logging
+os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
+logging.getLogger('tensorflow').setLevel(logging.ERROR)
 
+#pylint: disable=locally-disabled,bare-except,ungrouped-imports
+try:
+    import tensorflow as tf
+    tf.autograph.set_verbosity(0)
+    from bigml.images.featurizers import ImageFeaturizer as Featurizer
+except:
+    pass
 
 LOGGER = logging.getLogger('BigML')
 
-DATE_FNS = {
-    "day-of-month": lambda x: x.day,
-    "day-of-week": lambda x: x.weekday() + 1,
-    "millisecond": lambda x: x.microsecond / 1000}
-
 NUMERIC = "numeric"
 
 
@@ -67,26 +74,27 @@ def parse_items(text, regexp):
     return [term.strip() for term in pattern.split(text)]
 
 
-def check_model_fields(model):
-    """Checks the model structure to see whether it contains the required
+def check_resource_fields(resource):
+    """Checks the resource structure to see whether it contains the required
     fields information
 
     """
-    inner_key = FIELDS_PARENT.get(get_resource_type(model), 'model')
-    if check_model_structure(model, inner_key):
-        model = model.get('object', model)
-        fields = model.get("fields", model.get(inner_key, {}).get('fields'))
-        input_fields = model.get("input_fields")
+    inner_key = FIELDS_PARENT.get(get_resource_type(resource), 'model')
+    if check_resource_structure(resource, inner_key):
+        resource = resource.get('object', resource)
+        fields = resource.get("fields",
+            resource.get(inner_key, {}).get('fields'))
+        input_fields = resource.get("input_fields")
         # models only need model_fields to work. The rest of resources will
         # need all fields to work
-        model_fields = list(model.get(inner_key, {}).get( \
+        model_fields = list(resource.get(inner_key, {}).get( \
             'model_fields', {}).keys())
         # fusions don't have input fields
         if input_fields is None and inner_key != "fusion":
             return False
         if not model_fields:
-            fields_meta = model.get('fields_meta', \
-                model.get(inner_key, {}).get('fields_meta', {}))
+            fields_meta = resource.get('fields_meta', \
+                resource.get(inner_key, {}).get('fields_meta', {}))
             try:
                 return fields_meta['count'] == fields_meta['total']
             except KeyError:
@@ -96,22 +104,27 @@ def check_model_fields(model):
         else:
             if fields is None:
                 return False
-            return all([field_id in list(fields.keys()) \
-                for field_id in model_fields])
+            return all(field_id in list(fields.keys()) \
+                for field_id in model_fields)
     return False
 
 
-def check_model_structure(model, inner_key=None):
-    """Checks the model structure to see if it contains all the
+def check_resource_structure(resource, inner_key=None):
+    """Checks the resource structure to see if it contains all the
     main expected keys
 
     """
     if inner_key is None:
-        inner_key = FIELDS_PARENT.get(get_resource_type(model), 'model')
-    return (isinstance(model, dict) and 'resource' in model and
-            model['resource'] is not None and
-            (('object' in model and inner_key in model['object']) or
-             inner_key in model))
+        inner_key = FIELDS_PARENT.get(get_resource_type(resource), 'model')
+    # for datasets, only checking the resource ID
+    if inner_key is None:
+        return (isinstance(resource, dict) and 'resource' in resource and
+            resource['resource'] is not None)
+    # for the rest of models
+    return (isinstance(resource, dict) and 'resource' in resource and
+            resource['resource'] is not None and
+            (('object' in resource and inner_key in resource['object']) or
+             inner_key in resource))
 
 
 def get_unique_terms(terms, term_forms, tag_cloud):
@@ -139,95 +152,31 @@ def get_unique_terms(terms, term_forms, tag_cloud):
     return list(terms_set.items())
 
 
-def get_datetime_subfields(fields):
-    """ From a dictionary of fields, returns another dictionary
-    with the subfields from each datetime field
-
-    """
-    subfields = {}
-    for fid, finfo in list(fields.items()):
-        if finfo.get('parent_optype', False) == 'datetime':
-            parent_id = finfo["parent_ids"][0]
-            subfield = {fid: finfo["datatype"]}
-            if parent_id in list(subfields.keys()):
-                subfields[parent_id].update(subfield)
-            else:
-                subfields[parent_id] = subfield
-    return subfields
-
-
-def expand_date(date, subfields, timeformats):
-    """ Retrieves all the values of the subfields from
-    a given date
-
-    """
-    expanded = {}
-    try:
-        parsed_date = chronos.parse(date, format_names=timeformats)
-    except ValueError:
-        return {}
-    for fid, ftype in list(subfields.items()):
-        date_fn = DATE_FNS.get(ftype)
-        if date_fn is not None:
-            expanded.update({fid: date_fn(parsed_date)})
-        else:
-            expanded.update({fid: getattr(parsed_date, ftype)})
-    return expanded
-
-
-def get_datetime_formats(fields):
-    """From a dictionary of fields, return another dictionary
-    with the time formats form each datetime field
-
-    """
-    timeformats = {}
-    for f_id, finfo in list(fields.items()):
-        if finfo.get('optype', False) == 'datetime':
-            timeformats[f_id] = finfo.get('time_formats', {})
-    return timeformats
-
-
-def add_expanded_dates(input_data, datetime_fields):
-    """Add the expanded dates in date_fields to the input_data
-    provided by the user (only if the user didn't specify it)
-
-    """
-    for index, value in list(datetime_fields.items()):
-        if index not in input_data:
-            input_data[index] = value
-    return input_data
-
-
 class ModelFields():
     """ A lightweight wrapper of the field information in the model, cluster
     or anomaly objects
 
     """
-
+    #pylint: disable=locally-disabled,no-member,access-member-before-definition
     def __init__(self, fields, objective_id=None, data_locale=None,
                  missing_tokens=None, categories=False,
-                 numerics=False):
+                 numerics=False, operation_settings=None, model_fields=None):
         if isinstance(fields, dict):
+            tmp_fields = copy.deepcopy(fields)
             try:
                 self.objective_id = objective_id
-                self.uniquify_varnames(fields)
-                self.inverted_fields = invert_dictionary(fields)
-                self.fields = {}
-                self.fields.update(fields)
+                self.uniquify_varnames(tmp_fields)
+                self.inverted_fields = invert_dictionary(tmp_fields)
+                self.fields = tmp_fields
                 if not (hasattr(self, "input_fields") and self.input_fields):
                     self.input_fields = [field_id for field_id, field in \
                         sorted(list(self.fields.items()),
                                key=lambda x: x[1].get("column_number")) \
                         if not self.objective_id or \
                         field_id != self.objective_id]
-                self.model_fields = {}
-                self.datetime_parents = []
-                self.model_fields.update({field_id: field for field_id, field \
-                    in self.fields.items() if field_id in self.input_fields \
-                    and self.fields[field_id].get("preferred", True)})
-                # if any of the model fields is a generated datetime field
-                # we need to add the parent datetime field
-                self.model_fields = self.add_datetime_parents()
+                self.featurizer = Featurizer(self.fields, self.input_fields,
+                                             out_fields=model_fields)
+                self.model_fields = self.featurizer.out_fields
                 self.data_locale = data_locale
                 self.missing_tokens = missing_tokens
                 if self.data_locale is None:
@@ -255,10 +204,23 @@ class ModelFields():
                         'optype'] == NUMERIC \
                         or (hasattr(self, "boosting") and self.boosting and \
                         self.boosting.get("objective_class") is None)
-
+                self.operation_settings = self._add_operation_settings(
+                    operation_settings)
             except KeyError:
                 raise Exception("Wrong field structure.")
 
+    def _add_operation_settings(self, operation_settings):
+        """Checks and adds the user-given operation settings """
+        if operation_settings is None:
+            return None
+        if self.regression:
+            raise ValueError("No operating settings are allowed"
+                             " for regressions")
+        return {setting: operation_settings[setting] for
+            setting in operation_settings.keys() if setting in
+            DEFAULT_OPERATION_SETTINGS
+        }
+
     def add_terms(self, categories=False, numerics=False):
         """Adds the terms information of text and items fields
 
@@ -339,39 +301,6 @@ class ModelFields():
             value = str(value, "utf-8")
         return None if value in self.missing_tokens else value
 
-    def expand_datetime_fields(self, input_data):
-        """Returns the values for all the subfields
-        from all the datetime fields in input_data
-
-        """
-        expanded = {}
-        timeformats = get_datetime_formats(self.fields)
-        subfields = get_datetime_subfields(self.fields)
-        for f_id, value in list(input_data.items()):
-            if f_id in subfields:
-                formats = timeformats.get(f_id, [])
-                expanded.update(expand_date(value, subfields[f_id], formats))
-        return expanded
-
-    def add_datetime_parents(self):
-        """Adding the fields information for the fields that generate other
-        datetime fields used in the model
-        """
-        subfields = get_datetime_subfields(self.fields)
-        for f_id in list(subfields.keys()):
-            self.model_fields[f_id] = self.fields[f_id]
-            self.datetime_parents.append(f_id)
-        return self.model_fields
-
-    def remove_parent_datetimes(self, input_data):
-        """Removes the parents of datetime fields
-
-        """
-        for f_id in self.datetime_parents:
-            if f_id in input_data:
-                del input_data[f_id]
-        return input_data
-
     def fill_numeric_defaults(self, input_data):
         """Fills the value set as default for numeric missing fields if user
         created the model with the default_numeric_value option
@@ -417,13 +346,16 @@ class ModelFields():
                     new_input[key] = value
                 else:
                     unused_fields.append(key)
-            datetime_fields = self.expand_datetime_fields(new_input)
-            new_input = add_expanded_dates(new_input, datetime_fields)
-            new_input = self.remove_parent_datetimes(new_input)
+            # feature generation (datetime and image features)
+            new_input = self.featurizer.extend_input(new_input)
             # we fill the input with the chosen default, if selected
             new_input = self.fill_numeric_defaults(new_input)
-            result = (new_input, unused_fields) if add_unused_fields else \
-                new_input
+            final_input = {}
+            for key, value in new_input.items():
+                if key in self.model_fields:
+                    final_input.update({key: value})
+            result = (final_input, unused_fields) if add_unused_fields else \
+                final_input
             return result
         LOGGER.error("Failed to read input data in the expected"
                      " {field:value} format.")
@@ -465,6 +397,7 @@ class ModelFields():
                     unique_terms[field_id] = [(input_data_field, 1)]
                 del input_data[field_id]
         # the same for items fields
+        #pylint: disable=locally-disabled,consider-using-dict-items
         for field_id in self.item_analysis:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
@@ -505,4 +438,4 @@ class ModelFields():
 
         """
         self_vars = vars(self)
-        dumps(self_vars)
+        return dumps(self_vars)
