@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2021 BigML
+# Copyright 2018-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a PCA to make projections locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your PCAs offline.
 
 Example usage (assuming that you have previously set up the BIGML_USERNAME
@@ -41,12 +41,19 @@ pca.projection({"petal length": 3, "petal width": 1,
 import logging
 import math
 
+try:
+    from pandas import DataFrame
+    PANDAS_READY = True
+except ImportError:
+    PANDAS_READY = False
 
 from bigml.api import FINISHED
 from bigml.api import get_status, get_api_connection, get_pca_id
 from bigml.util import cast, use_cache, load, NUMERIC
 from bigml.basemodel import get_resource_dict
 from bigml.modelfields import ModelFields
+from bigml.constants import OUT_NEW_FIELDS, OUT_NEW_HEADERS
+
 
 try:
     from bigml.laminar.numpy_ops import dot
@@ -96,6 +103,7 @@ class PCA(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.input_fields = []
         self.default_numeric_value = None
         self.term_forms = {}
@@ -117,6 +125,7 @@ class PCA(ModelFields):
         if 'object' in pca and \
             isinstance(pca['object'], dict):
             pca = pca['object']
+            self.dataset_id = pca.get('dataset')
         try:
             self.input_fields = pca.get("input_fields", [])
             self.default_numeric_value = pca.get("default_numeric_value")
@@ -313,3 +322,50 @@ class PCA(ModelFields):
                 input_array.extend(new_inputs)
 
         return input_array, missings, input_mask
+
+    def batch_predict(self, input_data_list, outputs=None, **kwargs):
+        """Creates a batch projection for a list of inputs using the local
+        topic model. Allows to define some output settings to
+        decide the fields to be added to the input_data (prediction,
+        probability, etc.) and the name that we want to assign to these new
+        fields. The outputs argument accepts a dictionary with keys
+        "output_fields", to contain a list of the prediction properties to add
+        (all principal components by default) and "output_headers", to
+        contain a list of the headers to be used when adding them (identical
+        to "output_fields" list, by default).
+
+        :param input_data_list: List of input data to be predicted
+        :type input_data_list: list or Panda's dataframe
+        :param dict outputs: properties that define the headers and fields to
+                             be added to the input data
+        :return: the list of input data plus the predicted values
+        :rtype: list or Panda's dataframe depending on the input type in
+                input_data_list
+        """
+        if outputs is None:
+            outputs = {}
+        if outputs.get(OUT_NEW_FIELDS) is None:
+            new_fields = ["PC%s" % index for index in
+                          range(1, len(self.eigenvectors) + 1)]
+        if outputs.get(OUT_NEW_HEADERS):
+            new_headers = outputs.get(OUT_NEW_HEADERS, new_fields)
+            if len(new_fields) > len(new_headers):
+                new_headers.expand(new_fields[len(new_headers):])
+            else:
+                new_headers = new_headers[0: len(new_fields)]
+        else:
+            new_headers = new_fields
+        dataframe = False
+        if PANDAS_READY and isinstance(input_data_list, DataFrame):
+            dataframe = True
+            inner_data_list = input_data_list.to_dict('records')
+        else:
+            inner_data_list = input_data_list[:]
+        for input_data in inner_data_list:
+            kwargs.update({"full": True})
+            prediction = self.projection(input_data, **kwargs)
+            for index, key in enumerate(new_fields):
+                input_data[new_headers[index]] = prediction[key]
+        if PANDAS_READY and dataframe:
+            return DataFrame.from_dict(inner_data_list)
+        return inner_data_list
