@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,6 +18,7 @@
 import json
 import os
 
+from zipfile import ZipFile
 from nose.tools import eq_, assert_almost_equal, assert_is_not_none
 from .world import world, res_filename
 from bigml.model import Model, cast_prediction
@@ -33,10 +34,16 @@ from bigml.linear import LinearRegression
 from bigml.supervised import SupervisedModel
 from bigml.fusion import Fusion
 from bigml.pca import PCA
+from bigml.supervised import SupervisedModel
 
 
 from .create_prediction_steps import check_prediction
 
+def extract_zip(input_zip):
+    input_zip=ZipFile(input_zip)
+    return {name: input_zip.read(name) for name in input_zip.namelist()}
+
+
 #@step(r'I retrieve a list of remote models tagged with "(.*)"')
 def i_retrieve_a_list_of_remote_models(step, tag):
     world.list_of_models = [ \
@@ -67,6 +74,18 @@ def i_retrieve_a_list_of_remote_linear_regressions(step, tag):
 def i_create_a_local_model_from_file(step, model_file):
     world.local_model = Model(res_filename(model_file))
 
+#@step(r'I create a local deepnet from a "(.*)" file$')
+def i_create_a_local_deepnet_from_zip_file(step, deepnet_file,
+                                           operation_settings=None):
+    zipped_files = extract_zip(res_filename(deepnet_file))
+    deepnet = json.loads(list(zipped_files.values())[0])
+    world.local_model = Deepnet(deepnet,
+                                operation_settings=operation_settings)
+
+#@step(r'I create a local supervised model from a "(.*)" file$')
+def i_create_a_local_supervised_model_from_file(step, model_file):
+    world.local_model = SupervisedModel(res_filename(model_file))
+
 
 #@step(r'I create a local model$')
 def i_create_a_local_model(step):
@@ -110,6 +129,12 @@ def i_create_a_local_prediction(step, data=None):
     data = json.loads(data)
     world.local_prediction = world.local_model.predict(data, full=True)
 
+#@step(r'I create a local images prediction for "(.*)"$')
+def i_create_a_local_regions_prediction(step, image_file=None):
+    if image_file is None:
+        return None
+    data = res_filename(image_file)
+    world.local_prediction = world.local_model.predict(data, full=True)
 
 #@step(r'I create a local prediction for "(.*)" in operating point "(.*)"$')
 def i_create_a_local_prediction_op(step, data=None, operating_point=None):
@@ -148,11 +173,17 @@ def i_create_a_local_ensemble_prediction(step, data=None):
     world.local_prediction = world.local_ensemble.predict(data)
 
 #@step(r'I create a local deepnet prediction for "(.*)"$')
-def i_create_a_local_deepnet_prediction(step, data=None):
+def i_create_a_local_deepnet_prediction(step, data=None, image_fields=None,
+                                        full=False):
     if data is None:
         data = "{}"
+    if image_fields is None:
+        image_fields = []
     data = json.loads(data)
-    world.local_prediction = world.local_model.predict(data)
+    for field in image_fields:
+        if field in data:
+            data[field] = res_filename(data[field])
+    world.local_prediction = world.local_model.predict(data, full=full)
 
 #@step(r'I create a local deepnet prediction with operating point for "(.*)"$')
 def i_create_a_local_deepnet_prediction_with_op(step, data=None,
@@ -213,7 +244,7 @@ def the_local_centroid_is(step, centroid, distance):
 
 #@step(r'I create a local anomaly detector$')
 def i_create_a_local_anomaly(step):
-    world.local_anomaly = Anomaly(world.anomaly['resource'])
+    world.local_anomaly = Anomaly(world.anomaly["resource"])
 
 
 #@step(r'I create a local anomaly score for "(.*)"$')
@@ -291,9 +322,14 @@ def the_local_prediction_confidence_is(step, confidence):
 
 
 #@step(r'the highest local prediction\'s confidence for "(.*)" is "(.*)"')
-def the_highest_local_prediction_confidence_is(step, input_data, confidence):
+def the_highest_local_prediction_confidence_is(
+    step, input_data, confidence, missing_strategy=None):
     input_data = json.loads(input_data)
-    local_confidence = world.local_model.predict_confidence(input_data)
+    kwargs = {}
+    if missing_strategy is not None:
+        kwargs.update({"missing_strategy": missing_strategy})
+    local_confidence = world.local_model.predict_confidence(input_data,
+                                                            **kwargs)
     if isinstance(local_confidence, dict):
         local_confidence = round(float(local_confidence["confidence"]), 4)
     else:
@@ -303,7 +339,7 @@ def the_highest_local_prediction_confidence_is(step, input_data, confidence):
 
 
 #@step(r'the local prediction is "(.*)"')
-def the_local_prediction_is(step, prediction):
+def the_local_prediction_is(step, prediction, precision=4):
     if (isinstance(world.local_prediction, list) or
         isinstance(world.local_prediction, tuple)):
         local_prediction = world.local_prediction[0]
@@ -317,12 +353,25 @@ def the_local_prediction_is(step, prediction):
             world.local_model.regression) or \
             (isinstance(world.local_model, MultiModel) and \
             world.local_model.models[0].regression):
-        local_prediction = round(float(local_prediction), 4)
-        prediction = round(float(prediction), 4)
+        local_prediction = round(float(local_prediction), precision)
+        prediction = round(float(prediction), precision)
         assert_almost_equal(local_prediction, float(prediction),
-                            places=5)
+                            places=precision)
     else:
-        eq_(local_prediction, prediction)
+        if isinstance(local_prediction, str):
+            eq_(local_prediction, prediction)
+        else:
+            if isinstance(prediction, str):
+                prediction = float(prediction)
+            eq_(round(local_prediction, precision),
+                round(float(prediction), precision))
+
+
+#@step(r'the local regions prediction is "(.*)"')
+def the_local_regions_prediction_is(step, prediction):
+    prediction = json.loads(prediction)
+    eq_(prediction, world.local_prediction)
+
 
 #@step(r'the local probabilities are "(.*)"')
 def the_local_probabilities_are(step, prediction):
@@ -351,6 +400,11 @@ def the_local_probability_is(step, probability):
     probability =  round(float(probability), 4)
     local_probability = world.local_prediction["probability"]
 
+def eq_local_and_remote_probability(step):
+    local_probability = str(round(world.local_prediction["probability"], 3))
+    remote_probability = str(round(world.prediction["probability"], 3))
+    assert_almost_equal(local_probability, remote_probability)
+
 #@step(r'I create a local multi model')
 def i_create_a_local_multi_model(step):
     world.local_model = MultiModel(world.list_of_models)
@@ -390,7 +444,8 @@ def i_create_a_local_logistic_model(step):
 
 #@step(r'I create a local deepnet model$')
 def i_create_a_local_deepnet(step):
-    world.local_model = Deepnet(world.deepnet['resource'])
+    world.local_model = Deepnet({"resource": world.deepnet['resource'],
+                                 "object": world.deepnet})
     if hasattr(world, "local_ensemble"):
         world.local_ensemble = None
 
