@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,10 +18,10 @@ import time
 import json
 import os
 from datetime import datetime
-from .world import world, res_filename, logged_wait
+from .world import world, res_filename
 from nose.tools import eq_, assert_less
 
-from .read_cluster_steps import i_get_the_cluster
+from .read_resource_steps import wait_until_status_code_is
 
 from bigml.api import HTTP_CREATED
 from bigml.api import HTTP_ACCEPTED
@@ -32,17 +32,18 @@ from bigml.cluster import Cluster
 
 
 #@step(r'I create a cluster$')
-def i_create_a_cluster(step):
-    dataset = world.dataset.get('resource')
-    resource = world.api.create_cluster(
-        dataset, {'seed': 'BigML',
-                  'cluster_seed': 'BigML',
-                  'k': 8})
-    world.status = resource['code']
-    eq_(world.status, HTTP_CREATED)
-    world.location = resource['location']
-    world.cluster = resource['object']
-    world.clusters.append(resource['resource'])
+def i_create_a_cluster(step, shared=None):
+    if shared is None or world.shared.get("cluster", {}).get(shared) is None:
+        dataset = world.dataset.get('resource')
+        resource = world.api.create_cluster(
+            dataset, {'seed': 'BigML',
+                      'cluster_seed': 'BigML',
+                      'k': 8})
+        world.status = resource['code']
+        eq_(world.status, HTTP_CREATED)
+        world.location = resource['location']
+        world.cluster = resource['object']
+        world.clusters.append(resource['resource'])
 
 #@step(r'I create a cluster from a dataset list$')
 def i_create_a_cluster_from_dataset_list(step):
@@ -71,23 +72,22 @@ def i_create_a_cluster_with_options(step, options):
 
 #@step(r'I wait until the cluster status code is either (\d) or (-\d) less than (\d+)')
 def wait_until_cluster_status_code_is(step, code1, code2, secs):
-    start = datetime.utcnow()
-    delta = int(secs) * world.delta
-    i_get_the_cluster(step, world.cluster['resource'])
-    status = get_status(world.cluster)
-    count = 0
-    while (status['code'] != int(code1) and
-           status['code'] != int(code2)):
-        count += 1
-        logged_wait(start, delta, count, "cluster")
-        assert_less((datetime.utcnow() - start).seconds, delta)
-        i_get_the_cluster(step, world.cluster['resource'])
-        status = get_status(world.cluster)
-    eq_(status['code'], int(code1))
+    world.cluster = wait_until_status_code_is(
+        code1, code2, secs, world.cluster)
+
 
 #@step(r'I wait until the cluster is ready less than (\d+)')
-def the_cluster_is_finished_in_less_than(step, secs):
-    wait_until_cluster_status_code_is(step, FINISHED, FAULTY, secs)
+def the_cluster_is_finished_in_less_than(step, secs, shared=None):
+    if shared is None or world.shared.get("cluster", {}).get(shared) is None:
+        wait_until_cluster_status_code_is(step, FINISHED, FAULTY, secs)
+        if shared is not None:
+            if "cluster" not in world.shared:
+                world.shared["cluster"] = {}
+            world.shared["cluster"][shared] = world.cluster
+    else:
+        world.cluster = world.shared["cluster"][shared]
+        print("Reusing %s" % world.cluster["resource"])
+
 
 #@step(r'I make the cluster shared')
 def make_the_cluster_shared(step):
@@ -148,3 +148,18 @@ def i_create_local_cluster_from_file(step, export_file):
 #@step(r'the cluster ID and the local cluster ID match')
 def check_cluster_id_local_id(step):
     eq_(world.local_cluster.resource_id, world.cluster["resource"])
+
+
+#@step(r'I clone cluster')
+def clone_cluster(step, cluster):
+    resource = world.api.clone_cluster(cluster,
+                                       {'project': world.project_id})
+    # update status
+    world.status = resource['code']
+    world.location = resource['location']
+    world.cluster = resource['object']
+    # save reference
+    world.clusters.append(resource['resource'])
+
+def the_cloned_cluster_is(step, cluster):
+    eq_(world.cluster["origin"], cluster)
