@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2021 BigML
+# Copyright 2017-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,10 +18,10 @@ import time
 import json
 import os
 from datetime import datetime
-from .world import world, logged_wait
+from .world import world
 from nose.tools import eq_
 
-from .read_configuration_steps import i_get_the_configuration
+from .read_resource_steps import wait_until_status_code_is
 
 from bigml.api import HTTP_CREATED
 from bigml.api import HTTP_ACCEPTED
@@ -44,7 +44,6 @@ def i_create_configuration(step, configurations):
 def i_update_configuration(step, changes):
     resource = world.api.update_configuration(
         world.configuration["resource"], changes)
-    print(resource)
     world.status = resource['code']
     eq_(world.status, HTTP_ACCEPTED)
     world.location = resource['location']
@@ -53,19 +52,9 @@ def i_update_configuration(step, changes):
 
 #@step(r'I wait until the configuration status code is either (\d) or (-\d) less than (\d+)')
 def wait_until_configuration_status_code_is(step, code1, code2, secs):
-    start = datetime.utcnow()
-    delta = int(secs) * world.delta
-    i_get_the_configuration(step, world.configuration['resource'])
-    status = get_status(world.configuration)
-    count = 0
-    while (status['code'] != int(code1) and
-           status['code'] != int(code2)):
-        count += 1
-        logged_wait(start, delta, count, "configuration")
-        assert_less((datetime.utcnow() - start).seconds, delta)
-        i_get_the_configuration(step, world.configuration['resource'])
-        status = get_status(world.configuration)
-    eq_(status['code'], int(code1))
+    world.configuration = wait_until_status_code_is(
+        code1, code2, secs, world.configuration)
+
 
 #@step(r'I wait until the configuration is ready less than (\d+)')
 def the_configuration_is_finished_in_less_than(step, secs):
