@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,10 +18,10 @@ import time
 import json
 import os
 from datetime import datetime
-from .world import world, res_filename, logged_wait
+from .world import world, res_filename
 from nose.tools import eq_, assert_less
 
-from .read_lda_steps import i_get_the_topic_model
+from .read_resource_steps import wait_until_status_code_is
 
 from bigml.api import HTTP_CREATED
 from bigml.api import HTTP_ACCEPTED
@@ -78,19 +78,9 @@ def i_update_topic_model_name(step, name):
 
 #@step(r'I wait until the topic model status code is either (\d) or (-\d) less than (\d+)')
 def wait_until_topic_model_status_code_is(step, code1, code2, secs):
-    start = datetime.utcnow()
-    delta = int(secs) * world.delta
-    i_get_the_topic_model(step, world.topic_model['resource'])
-    status = get_status(world.topic_model)
-    count = 0
-    while (status['code'] != int(code1) and
-           status['code'] != int(code2)):
-        count += 1
-        logged_wait(start, delta, count, "topicmodel")
-        assert_less((datetime.utcnow() - start).seconds, delta)
-        i_get_the_topic_model(step, world.topic_model['resource'])
-        status = get_status(world.topic_model)
-    eq_(status['code'], int(code1))
+    world.topic_model = wait_until_status_code_is(
+        code1, code2, secs, world.topic_model)
+
 
 #@step(r'I wait until the topic model is ready less than (\d+)')
 def the_topic_model_is_finished_in_less_than(step, secs):
@@ -163,3 +153,17 @@ def i_create_local_topic_model_from_file(step, export_file):
 #@step(r'the topic model ID and the local topic model ID match')
 def check_topic_model_id_local_id(step):
     eq_(world.local_topic_model.resource_id, world.topic_model["resource"])
+
+#@step(r'I clone topic model')
+def clone_topic_model(step, topic_model):
+    resource = world.api.clone_topic_model(topic_model,
+                                           {'project': world.project_id})
+    # update status
+    world.status = resource['code']
+    world.location = resource['location']
+    world.topic_model = resource['object']
+    # save reference
+    world.topic_models.append(resource['resource'])
+
+def the_cloned_topic_model_is(step, topic_model):
+    eq_(world.topic_model["origin"], topic_model)
