@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,7 +18,7 @@ import time
 import json
 import os
 from datetime import datetime
-from .world import world, logged_wait
+from .world import world
 from nose.tools import eq_, assert_less
 
 from bigml.api import HTTP_CREATED
@@ -27,7 +27,7 @@ from bigml.api import FINISHED
 from bigml.api import FAULTY
 from bigml.api import get_status
 
-from .read_statistical_tst_steps import i_get_the_tst
+from .read_resource_steps import wait_until_status_code_is
 
 
 #@step(r'the statistical test name is "(.*)"')
@@ -59,20 +59,8 @@ def i_update_tst_name(step, name):
 
 #@step(r'I wait until the statistical test status code is either (\d) or (-\d) less than (\d+)')
 def wait_until_tst_status_code_is(step, code1, code2, secs):
-    start = datetime.utcnow()
-    delta = int(secs) * world.delta
-    statistical_test_id = world.statistical_test['resource']
-    i_get_the_tst(step, statistical_test_id)
-    status = get_status(world.statistical_test)
-    count = 0
-    while (status['code'] != int(code1) and
-           status['code'] != int(code2)):
-        count += 1
-        logged_wait(start, delta, count, "statisticaltests")
-        assert_less((datetime.utcnow() - start).seconds, delta)
-        i_get_the_tst(step, statistical_test_id)
-        status = get_status(world.statistical_test)
-    eq_(status['code'], int(code1))
+    world.statistical_test = wait_until_status_code_is(
+        code1, code2, secs, world.statistical_test)
 
 
 #@step(r'I wait until the statistical test is ready less than (\d+)')
