@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,7 +18,10 @@
 """ Creating test
 
 """
-from .world import world, setup_module, teardown_module
+import sys
+
+from .world import world, setup_module, teardown_module, show_doc, \
+    show_method, delete_local
 from . import create_source_steps as source_create
 from . import create_dataset_steps as dataset_create
 from . import create_statistical_tst_steps as statistical_tst_create
@@ -35,6 +38,7 @@ class TestStatisticalTest(object):
         """
             Debug information
         """
+        delete_local()
         print("\nEnd of tests in: %s\n-------------------\n" % __name__)
 
     def test_scenario1(self):
@@ -48,23 +52,34 @@ class TestStatisticalTest(object):
                 And I wait until the statistical test is ready less than <time_3> secs
                 And I update the statistical test name to "<test_name>"
                 When I wait until the statistical test is ready less than <time_4> secs
-                Then the statistical test name is "<correlation_name>"
+                Then the statistical test name is "<test_name>"
 
                 Examples:
                 | data                | time_1  | time_2 | time_3 | time_4 | test_name |
                 | ../data/iris.csv | 10      | 10     | 20     | 20 | my new statistical test name |
         """
-        print(self.test_scenario1.__doc__)
+        show_doc(self.test_scenario1)
+        headers = ["data", "source_wait", "dataset_wait", "model_wait",
+                   "test_name"]
         examples = [
-            ['data/iris.csv', '10', '10', '20', '20', 'my new statistical test name']]
+            ['data/iris.csv', '10', '10', '20', '20',
+             'my new statistical test name']]
         for example in examples:
-            print("\nTesting with:\n", example)
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
+            example = dict(zip(headers, example))
+            show_method(self, sys._getframe().f_code.co_name, example)
+            source_create.i_upload_a_file(
+                self, example["data"], shared=example["data"])
+            source_create.the_source_is_finished(
+                self, example["source_wait"], shared=example["data"])
+            dataset_create.i_create_a_dataset(self, shared=example["data"])
+            dataset_create.the_dataset_is_finished_in_less_than(
+                self, example["dataset_wait"], shared=example["data"])
             statistical_tst_create.i_create_a_tst_from_dataset(self)
-            statistical_tst_create.the_tst_is_finished_in_less_than(self, example[3])
-            statistical_tst_create.i_update_tst_name(self, example[5])
-            statistical_tst_create.the_tst_is_finished_in_less_than(self, example[4])
-            statistical_tst_create.i_check_tst_name(self, example[5])
+            statistical_tst_create.the_tst_is_finished_in_less_than(
+                self, example["model_wait"])
+            statistical_tst_create.i_update_tst_name(
+                self, example["test_name"])
+            statistical_tst_create.the_tst_is_finished_in_less_than(
+                self, example["model_wait"])
+            statistical_tst_create.i_check_tst_name(
+                self, example["test_name"])
