@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2015-2021 BigML
+# Copyright 2015-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,7 +18,10 @@
 """ Creating and updating scripts
 
 """
-from .world import world, setup_module, teardown_module
+import sys
+
+from .world import world, setup_module, teardown_module, show_doc, \
+    show_method, delete_local
 from . import create_script_steps as script_create
 
 class TestScript(object):
@@ -33,28 +36,31 @@ class TestScript(object):
         """
             Debug information
         """
+        delete_local()
         print("\nEnd of tests in: %s\n-------------------\n" % __name__)
 
     def test_scenario1(self):
         """
             Scenario: Successfully creating a whizzml script:
                 Given I create a whizzml script from a excerpt of code "<source_code>"
-                And I wait until the script is ready less than <time_1> secs
+                And I wait until the script is ready less than <script_wait> secs
                 And I update the script with "<param>", "<param_value>"
-                And I wait until the script is ready less than <time_2> secs
+                And I wait until the script is ready less than <script_wait> secs
                 Then the script code is "<source_code>" and the value of "<param>" is "<param_value>"
-
-                Examples:
-                | source_code      | time_1  | time_2  | param | param_value
-                | (+ 1 1)          | 10      | 10      | name  | my script
         """
-        print(self.test_scenario1.__doc__)
+        show_doc(self.test_scenario1)
+        headers = ["source_code", "script_wait", "param", "param_value"]
         examples = [
-            ['(+ 1 1)', '10', '10', 'name', 'my script']]
+            ['(+ 1 1)', '30', 'name', 'my script']]
         for example in examples:
-            print("\nTesting with:\n", example)
-            script_create.i_create_a_script(self, example[0])
-            script_create.the_script_is_finished(self, example[1])
-            script_create.i_update_a_script(self, example[3], example[4])
-            script_create.the_script_is_finished(self, example[2])
-            script_create.the_script_code_and_attributes(self, example[0], example[3], example[4])
+            example = dict(zip(headers, example))
+            show_method(self, sys._getframe().f_code.co_name, example)
+            script_create.i_create_a_script(self, example["source_code"])
+            script_create.the_script_is_finished(self, example["script_wait"])
+            script_create.i_update_a_script(
+                self, example["param"], example["param_value"])
+            script_create.the_script_is_finished(self, example["script_wait"])
+            script_create.the_script_code_and_attributes(
+                self, example["source_code"],
+                example["param"],
+                example["param_value"])
