@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2021 BigML
+# Copyright 2017-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ This module defines a Time Series model to make predictions locally or
 embedded into your application without needing to send requests to
 BigML.io.
 
-This module cannot only save you a few credits, but also enormously
+This module can help you enormously to
 reduce the latency for each prediction and let you use your logistic
 regressions offline.
 
@@ -141,6 +141,7 @@ class TimeSeries(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.input_fields = []
         self.default_numeric_value = None
         self.objective_fields = []
@@ -162,6 +163,7 @@ class TimeSeries(ModelFields):
         if 'object' in time_series and \
             isinstance(time_series['object'], dict):
             time_series = time_series['object']
+            self.dataset_id = time_series.get('dataset')
         try:
             self.input_fields = time_series.get("input_fields", [])
             self.default_numeric_value = time_series.get( \
