@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2016-2021 BigML
+# Copyright 2016-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -44,12 +44,18 @@ except ImportError:
     raise ImportError("Failed to import the Stemmer module. You need to"
                       " install pystemmer to use the Topic Model class.")
 
+try:
+    from pandas import DataFrame
+    PANDAS_READY = True
+except ImportError:
+    PANDAS_READY = False
 
 from bigml.api import FINISHED
 from bigml.api import get_status, get_api_connection, get_topic_model_id
 from bigml.basemodel import get_resource_dict
 from bigml.modelfields import ModelFields
 from bigml.util import use_cache, load, dump, dumps
+from bigml.constants import OUT_NEW_FIELDS, OUT_NEW_HEADERS
 
 
 LOGGER = logging.getLogger('BigML')
@@ -84,7 +90,7 @@ class TopicModel(ModelFields):
     to generate topic distributions for input documents locally.
 
     """
-
+    #pylint: disable=locally-disabled,c-extension-no-member,invalid-name
     def __init__(self, topic_model, api=None, cache_get=None):
 
         self.lang = None
@@ -97,6 +103,7 @@ class TopicModel(ModelFields):
             return
 
         self.resource_id = None
+        self.dataset_id = None
         self.seed = None
         self.case_sensitive = False
         self.bigrams = False
@@ -112,7 +119,7 @@ class TopicModel(ModelFields):
 
         if 'object' in topic_model and isinstance(topic_model['object'], dict):
             topic_model = topic_model['object']
-
+            self.dataset_id = topic_model.get('dataset')
         if 'topic_model' in topic_model \
                 and isinstance(topic_model['topic_model'], dict):
             status = get_status(topic_model)
@@ -376,6 +383,57 @@ class TopicModel(ModelFields):
         return [(sample_counts[k] + self.alpha) / normalizer
                 for k in range(self.ntopics)]
 
+    def batch_predict(self, input_data_list, outputs=None, **kwargs):
+        """Creates a batch prediction for a list of inputs using the local
+        supervised model. Allows to define some output settings to
+        decide the fields to be added to the input_data (prediction,
+        probability, etc.) and the name that we want to assign to these new
+        fields. The outputs argument accepts a dictionary with keys
+        "output_fields", to contain a list of the prediction properties to add
+        (["prediction", "probability"] by default) and "output_headers", to
+        contain a list of the headers to be used when adding them (identical
+        to "output_fields" list, by default).
+
+        :param input_data_list: List of input data to be predicted
+        :type input_data_list: list or Panda's dataframe
+        :param dict outputs: properties that define the headers and fields to
+                             be added to the input data
+        :return: the list of input data plus the predicted values
+        :rtype: list or Panda's dataframe depending on the input type in
+                input_data_list
+
+        """
+        if outputs is None:
+            outputs = {}
+        if outputs.get(OUT_NEW_FIELDS) is None:
+            new_fields = [topic["name"] for topic in self.topics]
+        if outputs.get(OUT_NEW_HEADERS):
+            new_headers = outputs.get(OUT_NEW_HEADERS, new_fields)
+            if len(new_fields) > len(new_headers):
+                new_headers.expand(new_fields[len(new_headers):])
+            else:
+                new_headers = new_headers[0: len(new_fields)]
+        else:
+            new_headers = new_fields
+        dataframe = False
+        if PANDAS_READY and isinstance(input_data_list, DataFrame):
+            dataframe = True
+            inner_data_list = input_data_list.to_dict('records')
+        else:
+            inner_data_list = input_data_list[:]
+        for index, input_data in enumerate(inner_data_list):
+            prediction = self.distribution(input_data, **kwargs)
+            prediction_dict = {}
+            for topic_distribution in prediction:
+                prediction_dict.update({topic_distribution["name"]:
+                    topic_distribution["probability"]})
+            for ikey, key in enumerate(new_fields):
+                inner_data_list[index][new_headers[ikey]] = prediction_dict[
+                    key]
+        if PANDAS_READY and dataframe:
+            return DataFrame.from_dict(inner_data_list)
+        return inner_data_list
+
     def dump(self, output=None, cache_set=None):
         """Uses msgpack to serialize the resource object
         If cache_set is filled with a cache set method, the method is called
