@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2021 BigML
+# Copyright 2012-2022 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -130,7 +130,7 @@ def localize(number):
     """Localizes `number` to show commas appropriately.
 
     """
-    return locale.format("%d", number, grouping=True)
+    return locale.format_string("%d", number, grouping=True)
 
 
 def is_url(value):
@@ -507,6 +507,7 @@ def save_json(resource, path):
         print("The resource has an invalid JSON format")
     except IOError:
         print("Failed writing resource to %s" % path)
+    return None
 
 
 def save(content, path):
@@ -554,14 +555,14 @@ def check_no_missing_numerics(input_data, fields, weight_field=None):
                              " data must contain values for all numeric"
                              " fields to get a prediction.")
 
-
+#pylint: disable=locally-disabled,too-many-boolean-expressions
 def check_no_training_missings(input_data, fields, weight_field=None,
                                objective_id=None):
     """Checks whether some input fields are missing in the input data
     while not training data has no missings in that field
 
     """
-    for field_id, field in list(fields.items()):
+    for field_id, field in fields.items():
         if (field["optype"] != "datetime" and \
                 field_id not in input_data and \
                 field['summary']['missing_count'] == 0 and \
@@ -621,3 +622,31 @@ def load(resource_id, cache_get):
     """
 
     return msgpack.loads(cache_get(resource_id))
+
+
+def filter_by_extension(file_list, extension_list):
+    """Returns the files that match the given extensions
+
+    """
+    return [filename for filename in file_list if
+        os.path.splitext(filename)[1].replace(".", "").lower()
+        in extension_list]
+
+
+def infer_field_type(field, value):
+    """Returns a dictionary containing the name and optype of the objective
+    field as inferred from the corresponding value
+    """
+    if isinstance(value, str):
+        optype = "categorical"
+    elif isinstance(value, list):
+        optype = "regions"
+    else:
+        optype = "numeric"
+    return {"name": field, "optype": optype}
+
+
+def is_image(filename):
+    """Checking whether the file is an image based on its extension """
+    return os.path.splitext(filename)[1].replace(".", "").lower() \
+        in c.IMAGE_EXTENSIONS
